!
! cabdiag --- CO-ADC for solids matrix diagonalization
! Christian Buth
! 16.06.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 16.06.2004 (Diagonalize ADC matrices with ARPACK)
!


!
! Program
!

      program cabdiag
      use physconst
      use lattice
      use wannier
      use coadcdiag
      use coadcfile
      implicit none

      logical fulldiag

      character(1024) fileADCip,      &                 ! Negative of ADC eigenvalues
     &                fileCOADCBands                    ! CO-ADC band structure

      integer numBrillouinPaths, niter
      integer, parameter :: ffockip    = 50, &
     &                      fadcip     = 51, &
     &                      ffockbands = 52, &
     &                      fadcbands  = 53

      real(dblprec) firstBand, lastBand, polethresh

      type(tcoadcmatrix) am


      namelist /coadc/ numBrillouinPaths, fulldiag, niter
     &               !numUnitCells, numWannierUnitCells, startOccupied, endVirtual, &
     &               !firstBand, lastBand, polethresh, periodicityDim, &
     &               !fileFockip, fileADCip, fileFockBands, fileCOADCBands, &
     &               !fileBands, fileFockAll, &
     &               !fileFockTrans, fileVBucket, fileVijkl, fileVIndex, &

      ! Greeting
      write (*,*) 'Crystal orbital algebraic diagrammatic constuction eigenvalue solver'
      write (*,*) 'Copyright (c) 2004-2006 Christian Buth'
      write (*,*)
      write (*,*)


      fulldiag = .true.
      firstBand = -1000
      lastBand = 1000
      polethresh = 0.02
      fileADCip = 'adcip.dat'
      fileCOADCBands = 'coadcbands.dat'
      niter = 500

      read (*,coadc)


      ! Open files for the diagonal and the off-diagonal matrix
      ! elements of the CO-ADC matrix
      write (*,*) 'Open files for the diagonal and the off-diagonal matrix elements', &
     &            'of the CO-ADC matrix'
      am%readfile = .true.
      am%fadccom  = 393
      am%fadcdia  = 394
      am%fadcoff  = 395
      call opencoadcfile(am)

      ! Carry out CO-ADC calculation
      select case (am%periodicity)
        case (0)
          write (*,*) 'The system is a molecule'
        case (1)
          write (*,*) 'The system is a polymer'
        case (2)
          write (*,*) 'The system is a surface'
        case (3)
          write (*,*) 'The system is a crystal'
      end select

      if (am%periodicity == 0) then
        call ipea(am)
      else
        call bandstruct(am, niter)
      end if

      call closecoadcfile(am)


      write (*,*)
      write (*,*)
      write (*,*)
      write (*,*) 'Excecution sucessful'
      write (*,*)
      write (*,*) 'Da steh ich nun, ich armer Tor!'
      write (*,*) 'Und bin so klug als wie zuvor;'
      write (*,*) '                Goethe, Faust I'


      contains

!
! ipea --- calculate the Hartree-Fock and ADC ionization potentials and electron affinities
!

      subroutine ipea(am)
      implicit none

      type(tcoadcmatrix), intent(inout) :: am
      integer i
      type(tbandsatkpoint) ad

      ! ADC ionization potentials and electron affinities
      open(unit=fadcip,file=fileADCip,status='replace',action='write')

      if (fulldiag) then
        call ipeafulldiag(am, ad)
      else
        call ipealanczosdiag(am, ad)
      end if

      do i=1, ad%numBands
        if (ad%energies(i) * HartreetoeV > firstBand .and. &
     &      ad%energies(i) * HartreetoeV <  lastBand .and. &
     &      ad%polestrengths(i) > polethresh) then
          write (fadcip,'(2F15.6)') -ad%energies(i) * HartreetoeV, ad%polestrengths(i)
        end if
      end do
      write (fadcip,*)

      close(unit=fadcip)
      write (*,*) ' ADC ionization potentials and electron affinities are in ',trim(fileADCip)

      deallocate(ad%energies, ad%polestrengths)
      end subroutine ipea


!
! bandstruct --- calculate the Hartree-Fock and CO-ADC band structures
!

      subroutine bandstruct(am, niter)
      use mathtl
      use bandstructure
      implicit none

      integer, intent(in) :: niter
      type(tcoadcmatrix), intent(inout) :: am
      integer i, j, numOutBands, currPath
      type(tbandsatkpoint) ad
      type(tlatticevector) kpoint
      type(tfbands) fbands

      ! Get paths though the Brillouin zone
      allocate(fbands%paths(numBrillouinPaths))
      fbands%fock = .false.
      fbands%numPaths = numBrillouinPaths
      fbands%startOccupied = 1
      fbands%numOccupied = am%nocc
      fbands%numOrbitals = am%main
      fbands%periodicity = am%periodicity

      do currPath=1, numBrillouinPaths
        call readbrillouinpathspec(fbands%paths(currPath), fstdin, fbands%periodicity, 0)
      end do

      call reportbrillouinpathspec(fbands%paths, numBrillouinPaths, &
     &          fbands%periodicity)

      ! Correlated band structure
      call openbands(fbands, fadcbands, fileCOADCBands, 'write')
      nullify(ad%energies, ad%polestrengths)

      do currPath=1, numBrillouinPaths
        call putbrillouinpath(fbands, currPath)
        kpoint = fbands%paths(currPath)%beg
        do i=1, fbands%paths(currPath)%numkpoints
          call newk(am%lattice, kpoint)
          if (fulldiag) then
            call bandfulldiag(am, ad)
          else
            call bandlanczosdiag(am, ad, niter)
          end if

          ! Count number of bands to be written
          numOutBands = 0
          do j=1, ad%numBands
            if (ad%energies(j) * HartreetoeV >= firstBand .and. &
     &          ad%energies(j) * HartreetoeV <=  lastBand .and. &
     &          ad%polestrengths(j) >= polethresh) then
              numOutBands = numOutBands + 1
            end if
          end do

          call putkpointheader(fbands, kpoint, numOutBands)

          ! Write band data
          do j=1, ad%numBands
            if (ad%energies(j) * HartreetoeV > firstBand .and. &
     &          ad%energies(j) * HartreetoeV <  lastBand .and. &
     &          ad%polestrengths(j) > polethresh) then
              call putbandpoint(fbands, ad%energies(j) * HartreetoeV, &
     &                          ad%polestrengths(j))
            end if
          end do

!          kpoint%a = kpoint%a + fbands%paths(currPath)%inc%a
!          kpoint%b = kpoint%b + fbands%paths(currPath)%inc%b
!          kpoint%c = kpoint%c + fbands%paths(currPath)%inc%c
          kpoint%v = kpoint%v + fbands%paths(currPath)%inc%v
          deallocate(ad%energies, ad%polestrengths)
        end do
      end do

      call closebands(fbands)
      write (*,*) ' CO-ADC band structure is in ', trim(fileCOADCBands)
      end subroutine bandstruct
      end program cabdiag
