!
! ipeamatvec --- matrix times block-vector for the Dyson ADC matrix
! Christian Buth
! 16.06.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 16.06.2004 (bandfulldiag taken as basis)
! Version 1.10: 24.08.2004 (bandmatvec taken as basis)
! Version 1.20: 26.08.2004 (Several bugs removed)
!

!
! Subroutine
!

      subroutine ipeamatvec(am, bw, vpre, vpost)
      use coadcfile
      use coadcdiag
      implicit none

      integer, intent(in) :: bw
      real(dblprec), intent(in) :: vpre(:,:)
      real(dblprec), intent(out) :: vpost(:,:)
      type(tcoadcmatrix), intent(inout) :: am
      logical ameof
      integer i, j

        write (*,*) ' am%main, bw'
        write (*,*) am%main, bw

      call amrewind(am)
      vpost = 0

      ! Multiply vector with diagonal elements of ADC matrix
      do
        call readmatrixbuffer(am, .true., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwd
          do j=1, bw
            ! ADC matrix is real symmetric
            vpost(am%idx(i), j) = am%diag(i) * vpre(am%idx(i), j)
          end do
        end do
      end do

      ! Multiply vector with off-diagonal elements of ADC matrix
      do
        call readmatrixbuffer(am, .false., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwo
          do j=1, bw
            ! ADC matrix is real symmetric where the upper triangle is stored
            vpost(am%jox(i), j) = vpost(am%jox(i), j)            &
     &                    + am%off(i) * vpre(am%iox(i), j)
            vpost(am%iox(i), j) = vpost(am%iox(i), j)            &
     &                    + am%off(i) * vpre(am%jox(i), j)
          end do
        end do
      end do
      end subroutine ipeamatvec
