!
! bandfulldiag --- Diagonalize band structure matrix fully
! Christian Buth
! 14.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 14.02.2004 (Diagonalize band structure matrix fully)
! Version 1.10: 28.02.2004 (Active orbitals can be specified for correlation calculations)
! Version 1.30: 04.03.2004 (Optional treatment of the non-diagonal Fock matrix
!       at CO-ADC(3) level)
! Version 1.40: 28.06.2004 (Matrix generation code replaced by matrix reader)
!


!
! Include files
!

#include "assert.h"


!
! Subroutine
!

      subroutine bandfulldiag(am, ad)
      use physconst
      use lattice
      use adcdiag
      use coadcfile
      implicit none

      type(tadcdiag), intent(inout) :: ad
      type(tcoadcmatrix), intent(inout) :: am

      logical ameof

      integer i, j,    &
     &        info, lwork, lrwork, numUpperTriangle, nonzero, numADCBands

      real sparsity

      real(dblprec) rx
      real(dblprec), allocatable :: rwork(:)

      complex(dblprec), allocatable :: work(:),        &
     &                                 bandstruct(:,:)

      numADCBands = am%ndim
      lwork = 2 * numADCBands - 1
      lrwork = 3 * numADCBands - 2

      ! Allocate memory
      ad%numStates = am%ndim
      allocate(bandstruct(numADCBands,numADCBands), &
     &         work(lwork),                         &
     &         rwork(lrwork))

      ! Get diagonal elements of CO-ADC matrix
      call amrewind(am)
      bandstruct = 0
      do
        call readmatrixbuffer(am, .true., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwd
          bandstruct(am%idx(i), am%idx(i)) = bandstruct(am%idx(i), am%idx(i)) &
     &               + am%diag(i) * conjg(am%lattice%factor(am%latd(i)))
        end do
      end do


      ! Get off-diagonal elements of CO-ADC matrix
      do
        call readmatrixbuffer(am, .false., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwo
          bandstruct(am%iox(i), am%jox(i)) = bandstruct(am%iox(i), am%jox(i)) &
     &          + am%off(i) * conjg(am%lattice%factor(am%lato(i)))
        end do
      end do


      ! Sparsity of the band structure matrix
      nonzero = 0
      do i=1, numADCBands
        do j=i, numADCBands
          if (bandstruct(i,j) /= 0) then
            nonzero = nonzero + 1
          end if
        end do
      end do
      ! The zero coupling matrix between 2p1h block and 2h1p block is subtracted
      !!!numUpperTriangle = numADCBands * (numADCBands + 1) / 2 - num2p1h * num2h1p
      numUpperTriangle = numADCBands * (numADCBands + 1) / 2
      sparsity = real(numUpperTriangle - nonzero) / real(numUpperTriangle) * 100.0

      write (*,'(A,F6.2,A)') 'Sparsity of ADC matrix ', sparsity, '%'


      write (*,*)
      write (*,*) 'Diagonalizing the band structure matrix  (sparsity = ', 100.0 - sparsity, '%)...'
      call zheev('V', 'U', numADCBands, bandstruct, numADCBands, ad%eigenvalues, work, lwork, rwork, info)

      if (info.ne.0) then
          write (*,*)
          write (*,*) 'zheev returned error while diagonalizing the Fock matrix: info = ', info
          stop 'CO-ADC execution error'
      end if

      ! Write ionization potentials
      write (*,*)
      write (*,*) 'The band structure at k-point ', am%lattice%crystalmomentum
      write (*,*) 'IP [a.u.]             IP [eV]             Pole strength'

      do i=1, numADCBands
        rx = 0
        do j=1, am%main
          rx = rx + bandstruct(j,i) * conjg(bandstruct(j,i))
        end do
        ad%polestrengths(i) = rx

        if (rx > 0.3) then
          write (*,*) ad%eigenvalues(i), ad%eigenvalues(i) * HartreetoeV, rx
        end if
      end do

      write (*,*)
      deallocate(bandstruct, work, rwork)
      end subroutine bandfulldiag
