!
! adcmatrix --- write ADC matrix to a file
! Christian Buth
! 13.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 13.02.2004 (Diagonalize the ADC matrix fully)
! Version 1.10: 29.02.2004 (Integrated into adcbands module)
! Version 1.20: 02.03.2004 (Optional exact treatment of the non-diagonal Fock matrix)
! Version 1.30: 03.03.2004 (Optional treatment of the non-diagonal Fock matrix at ADC(3) level)
! Version 1.40: 15.06.2004 (Do not diagonalize ADC matrix but write to a file)
! Version 1.50: 26.06.2004 (Write ADC matrix for an exact treatment of the non-diagonal Fock matrix
!       and a treatment of the non-diagonal Fock matrix at third order)
!


!
! Include files
!

#include "assert.h"


!
! Subroutine
!

      subroutine adcmatrix(wb, cm, fockeigenvectors, fockeigenvalues, &
     &           exactoffdiag,fockpt3)
      use wannier
      use adcbands
      use coadcfile
      implicit none

      logical, intent(in) :: exactoffdiag, fockpt3
      integer i, j, R
      real(dblprec), intent(in) :: fockeigenvectors(:,:), &            ! Eigenvectors of the Fock matrix
     &                             fockeigenvalues(:)                  ! Eigenvalues of the Fock matrix

      real(dblprec) rx
      type(twannier), intent(in) :: wb
      type(tcoadc), intent(in) :: cm
      type(tcoadcmatrix) am
      type(tconfigurationset) cfg2p1h, cfg2h1p


      write (*,*) 'Generating the ADC matrix...'
      write (*,*) '    Number of 2p1h configurations: ', cm%num2p1h
      write (*,*) '    Number of 2h1p configurations: ', cm%num2h1p

      ! Open files for the diagonal and the off-diagonal matrix
      ! elements of the CO-ADC matrix
      am%readfile     = .false.
      am%fadccom      = 393
      am%fadcdia      = 394
      am%fadcoff      = 395
      am%periodicity  = wb%periodicity
      am%main         = wb%numOrbitals
      am%nocc         = wb%numOccupied
      am%ndim         = cm%numBands
      am%nbuf         = 10000
      am%threshold    = 0
      ! Usually in three-dimensional systems, far more unit cells are needed to
      ! represent the FOCKALL matrix than are in the Wannier cluster. For
      ! one-dimensional systems, sometimes the number of FOCKALL unit cells
      ! is smaller than the number of cells in the Wannier cluster.
      ! am%lattice%size = max(wb%numFockAllUnitCells, wb%numClusterUnitCells)
      am%lattice%size = wb%numFockAllUnitCells
      if (cm%numUnitCells > wb%numFockAllUnitCells) then
        write (*,*) 'You request me to correlate electrons in more unit &
                    &cells than are needed to represent the FOCKALL &
                    &matrix! Do you think this is reasonable? If you do &
                    &I need more lattice vectors, so try to get some &
                    &for me!'
        stop 'cm%numUnitCells > wb%numFockAllUnitCells'
      end if
      call opencoadcfile(am)

      if (wb%periodicity > 0) then
        do i=1, wb%numFockAllUnitCells
          am%lattice%idirect(i) = wb%idirect(i)
          am%lattice%cdirect(i) = wb%cdirect(i)
        end do
      end if

      ! Write Fock matrix
      write (*,*) '    Generating Fock matrix'
      do i=wb%startOccupied, wb%endVirtual
        do R=1, wb%numFockAllUnitCells
          !!rx = ffind(i, i)
          rx = fallfind(i, R, i)
          call writediagentry(am, i, R, rx)
        end do

        do j=i+1, wb%endVirtual
          do R=1, wb%numFockAllUnitCells
            !!rx = ffind(i, j)
            rx = fallfind(i, R, j)
            call writeoffdiagentry(am, i, j, R, rx)
          end do
        end do
      end do


  !      fockpt3 = .true.
  !      fockpt3 = .false.
  !      exactoffdiag = .true.
  !      exactoffdiag = .false.


      write (*,*) '    Generating 2p1h Block'
      call selectconfigs(cfg2p1h, .true.)
      call COADC_2_2_single_triple(am, cfg2p1h, .true.)
      if (exactoffdiag) then
        call COADC_2_infty_triple_triple(am, cfg2p1h, .true.)
      else if (fockpt3) then
        call COADC_2_3_triple_triple(am, cfg2p1h, .true.)
      else
        call COADC_2_2_triple_triple(am, cfg2p1h, .true.)
      end if


      write (*,*) '    Generating 2h1p Block'
      call selectconfigs(cfg2h1p, .false.)
      call COADC_2_2_single_triple(am, cfg2h1p, .false.)
      if (exactoffdiag) then
        call COADC_2_infty_triple_triple(am, cfg2h1p, .false.)
      else if (fockpt3) then
        call COADC_2_3_triple_triple(am, cfg2h1p, .false.)
      else
        call COADC_2_2_triple_triple(am, cfg2h1p, .false.)
      end if


/*********
      ! Sparsity of the band structure matrix
      nonzero = 0
      do i=1, numADCBands
        do j=i, numADCBands
          if (bandstruct(i,j) /= 0) then
            nonzero = nonzero + 1
          end if
        end do
      end do
      minNonZero = numADCBands * (numADCBands + 1) / 2 - 2 * num2p1h * num2h1p / 2
      sparsity = real(nonzero) / real(minNonZero) * 100.0

*******/

      call closecoadcfile(am)
      deallocate(cfg2p1h%configs, cfg2h1p%configs)


      contains

!
! selectconfigs --- select most important 2p1h or 2h1p configurations used to build
!       CO-ADC matrix
!

      subroutine selectconfigs(cfgset, do2p1h)
      implicit none

      logical do2p1h

      integer j, k, l, g1, g2, idx, &
     &        norbs1, norbs2, shiftj, shiftk, shiftl

      type(tconfigurationset) cfgset

      if (do2p1h) then
        norbs1 = cm%numOccupied
        norbs2 = cm%numVirtual
        shiftj = wb%numOccupied
        shiftk = wb%numOccupied
        shiftl = cm%startOccupied - 1
      else
        norbs1 = cm%numVirtual
        norbs2 = cm%numOccupied
        shiftj = cm%startOccupied - 1
        shiftk = cm%startOccupied - 1
        shiftl = wb%numOccupied
      end if

      cfgset%numConfigs = norbs1 * norbs2 * (norbs2 + 1) / 2 * cm%numUnitCells**2

      allocate(cfgset%configs(cfgset%numConfigs))
      write (*,*) 'Selected configurations: ', cfgset%numConfigs

      idx = 1
      do l=1, norbs1
        do k=1, norbs2
          do j=1, k
            do g1=1, cm%numUnitCells
              do g2=1, cm%numUnitCells

                cfgset%configs(idx)%g1 = g1
                cfgset%configs(idx)%g2 = g2
                cfgset%configs(idx)%cj = j + shiftj
                cfgset%configs(idx)%ck = k + shiftk
                cfgset%configs(idx)%cl = l + shiftl
                idx = idx + 1

              end do
            end do
          end do
        end do
      end do

      assert((idx-1) == cfgset%numConfigs)
      end subroutine selectconfigs


!
! epsilontensor --- transform the diagonal elements of the ADC(2)
!       matrix to the corresponding matrix elements for a non-diagonal Fock matrix
!

      real(dblprec) function epsilontensor(rcfg, ccfg, do2p1h)
      implicit none

      logical, intent(in) :: do2p1h
      integer rj, rk, rl, cj, ck, cl    !, &
!     &        j, k, l
      type(tconfiguration), intent(in) :: rcfg, ccfg

      rj = rcfg%cj
      rk = rcfg%ck
      rl = rcfg%cl
      cj = ccfg%cj
      ck = ccfg%ck
      cl = ccfg%cl

      if (do2p1h) then

        epsilontensor = fallfind(rj, 1, cj) + fallfind(rk, 1, ck) - fallfind(rl, 1, cl)
        stop 'development in progress: adcmatrix'

      else



      end if

/***********************************************************************************************

        Old style implementation, to assure correctness

      if (do2p1h) then
        epsilontensor = epsilontensor2p1h(ccfg, rcfg)
!!        epsilontensor = epsilontensor2p1h(rcfg, ccfg)
      else
!! scheint keinen effekt zu haben
        epsilontensor = epsilontensor2h1p(ccfg, rcfg)
!!        epsilontensor = epsilontensor2h1p(rcfg, ccfg)
      end if
***********************************************************************************************/
      end function epsilontensor


!
! epsilontensor2p1h --- transform the eps_a + eps_b - eps_i diagonal elements of the ADC(2)
!       matrix to the corresponding matrix elements for a non-diagonal Fock matrix
!

      real(dblprec) function epsilontensor2p1h(rcfg, ccfg)
      implicit none

      integer rj, rk, rl, cj, ck, cl, &
     &        j, k, l

      real(dblprec) me

      type(tconfiguration) rcfg, ccfg

      rj = rcfg%cj
      rk = rcfg%ck
      rl = rcfg%cl
      cj = ccfg%cj
      ck = ccfg%ck
      cl = ccfg%cl

      me = 0
      do j=1, wb%numVirtual
        do k=1, wb%numVirtual
          do l=1, wb%numOccupied
            me = me +  fockeigenvectors(rj,  j + wb%numOccupied) &
     &               * fockeigenvectors(rk,  k + wb%numOccupied) &
     &               * fockeigenvectors(rl,  l)  &
     &                  * (  fockeigenvalues(j + wb%numOccupied) &
     &                     + fockeigenvalues(k + wb%numOccupied) &
     &                     - fockeigenvalues(l)) &
     &               * fockeigenvectors(cj,  j + wb%numOccupied) &
     &               * fockeigenvectors(ck,  k + wb%numOccupied) &
     &               * fockeigenvectors(cl,  l)
          end do
        end do
      end do

      epsilontensor2p1h = me
      end function epsilontensor2p1h


!
! epsilontensor2h1p --- transform the eps_i + eps_j - eps_a diagonal elements of the ADC(2)
!       matrix to the corresponding matrix elements for a non-diagonal Fock matrix
!

      real(dblprec) function epsilontensor2h1p(rcfg, ccfg)
      implicit none

      integer rj, rk, rl, cj, ck, cl

      integer j, k, l

      real(dblprec) me

      type(tconfiguration) rcfg, ccfg

      rj = rcfg%cj
      rk = rcfg%ck
      rl = rcfg%cl
      cj = ccfg%cj
      ck = ccfg%ck
      cl = ccfg%cl

      me = 0
      do j=1, wb%numOccupied
        do k=1, wb%numOccupied
          do l=1, wb%numVirtual
            me = me +  fockeigenvectors(rj,  j)   &
     &               * fockeigenvectors(rk,  k)   &
     &               * fockeigenvectors(rl,  l + wb%numOccupied)  &
     &                  * (  fockeigenvalues(j)   &
     &                     + fockeigenvalues(k)   &
     &                     - fockeigenvalues(l + wb%numOccupied)) &
     &               * fockeigenvectors(cj,  j)   &
     &               * fockeigenvectors(ck,  k)   &
     &               * fockeigenvectors(cl,  l + wb%numOccupied)
          end do
        end do
      end do

      epsilontensor2h1p = me
      end function epsilontensor2h1p


!
! cfgcoulomb --- get two-electron coulomb integral for an orbital coupling to a
!       2p1h/2h1p configuration
!

      real(dblprec) function cfgcoulomb(R, i, cfg)
      implicit none

      integer R, i

      type(tconfiguration) cfg

      cfgcoulomb = gfind(wannierOrbital(wb,      R,      i), &
     &                   wannierOrbital(wb,      1, cfg%cl), &
     &                   wannierOrbital(wb, cfg%g1, cfg%cj), &
     &                   wannierOrbital(wb, cfg%g2, cfg%ck))
      end function cfgcoulomb


!
! cfgexchange --- get two-electron exchange integral for an orbital coupling to a
!       2p1h/2h1p configuration
!

      real(dblprec) function cfgexchange(R, i, cfg)
      implicit none

      integer R, i

      type(tconfiguration) cfg

      cfgexchange = gfind(wannierOrbital(wb,      R,      i), &
     &                    wannierOrbital(wb,      1, cfg%cl), &
     &                    wannierOrbital(wb, cfg%g2, cfg%ck), &
     &                    wannierOrbital(wb, cfg%g1, cfg%cj))
      end function cfgexchange


!
! COADC_2_2_single_triple --- 1h,1p/2p1h or 1h,1p/2h1p coupling block in CO-ADC, second
!       order for the Coulomb interaction and second order for the
!       off-diagonal of the Fock matrix
!

      subroutine COADC_2_2_single_triple(am, cfgset, do2p1h)
      implicit none

      logical, intent(in) :: do2p1h
      type(tcoadcmatrix), intent(inout) :: am
      type(tconfigurationset), intent(in) :: cfgset
      integer R, i, ci, cidx
      real(dblprec) rV1, rV2

      ! All columns of the 1h,1p/2p1h block
      if (do2p1h) then
        cidx = wb%numOrbitals + 1
      ! All columns of the 1h,1p/2h1p block
      else
        cidx = wb%numOrbitals + cm%num2p1h + 1
      end if

      do ci=1, cfgset%numConfigs

        ! Case g1 == g2 and cj == ck for a column of the 2p1h or 2h1p matrix
        if (cfgset%configs(ci)%g1 == cfgset%configs(ci)%g2 .and. &
     &      cfgset%configs(ci)%cj == cfgset%configs(ci)%ck) then

          do i=1, wb%numOrbitals
            do R = 1, cm%numUnitCells
              rV1 = cfgcoulomb(R, i, cfgset%configs(ci))
              call writeoffdiagentry(am, i, cidx, R, rV1)
            end do
          end do
          cidx = cidx + 1

        ! Case g1 /= g2 or cj /= ck for a column of the 2p1h or 2h1p matrix
        else

          do i=1, wb%numOrbitals
            do R = 1, cm%numUnitCells
              rV1 = cfgcoulomb(R, i, cfgset%configs(ci))
              rV2 = cfgexchange(R, i, cfgset%configs(ci))
              call writeoffdiagentry(am, i, cidx    , R, (rV1 + rV2) * sqrt(1.0D0 / 2.0D0))
              call writeoffdiagentry(am, i, cidx + 1, R, (rV1 - rV2) * sqrt(3.0D0 / 2.0D0))
            end do
          end do
          cidx = cidx + 2

        end if
      end do

#ifndef NDEBUG
      if (do2p1h) then
        write (*,*) 'cidx ', cidx-1, 'num2p1h+numWannierOrbitals', cm%num2p1h+wb%numOrbitals
      else
        write (*,*) 'cidx ', cidx-1, 'numADCBands', cm%numBands
      end if
#endif
      end subroutine COADC_2_2_single_triple


!
! COADC_2_2_triple_triple --- 2p1h/2p1h or 2h1p/2h1p block in CO-ADC, second order for
!       the Coulomb interaction and second order for the off-diagonal
!       of the Fock matrix
!

      subroutine COADC_2_2_triple_triple(am, cfgset, do2p1h)
      implicit none

      logical, intent(in) :: do2p1h
      type(tcoadcmatrix), intent(inout) :: am
      type(tconfigurationset), intent(in) :: cfgset
      integer cj, ck, cl, ri, ridx
      real(dblprec) rx


      ! All rows of the 2p1h/2p1h block
      if (do2p1h) then
        ridx = wb%numOrbitals + 1
      ! All rows of the 2h1p/2h1p block
      else
        ridx = wb%numOrbitals + cm%num2p1h + 1
      end if


      do ri=1, cfgset%numConfigs

        cj = cfgset%configs(ri)%cj
        ck = cfgset%configs(ri)%ck
        cl = cfgset%configs(ri)%cl

        ! Case g1 == g2 and cj == ck for a column of the 2p1h or 2h1p matrix
        if (cfgset%configs(ri)%g1 == cfgset%configs(ri)%g2 .and. cj == ck) then

          rx = wannierepsilon(cj) + wannierepsilon(cj) - wannierepsilon(cl)
          call writediagentry(am, ridx, 1, rx)
          ridx = ridx + 1

        ! Case g1 /= g2 or cj /= ck for a column of the 2p1h or 2h1p matrix
        else

          rx = wannierepsilon(cj) + wannierepsilon(ck) - wannierepsilon(cl)
          call writediagentry(am, ridx  , 1, rx)
          call writediagentry(am, ridx+1, 1, rx)
          ridx = ridx + 2

        end if
      end do
      end subroutine COADC_2_2_triple_triple


!
! COADC_2_3_triple_triple --- 2p1h/2p1h or 2h1p/2h1p block in CO-ADC, second order for
!       the Coulomb interaction and third order for the off-diagonal
!       of the Fock matrix
!

      subroutine COADC_2_3_triple_triple(am, cfgset, do2p1h)
      implicit none

      logical, intent(in) :: do2p1h
      type(tcoadcmatrix), intent(inout) :: am
      type(tconfigurationset), intent(in) :: cfgset
      integer rg1, rg2, rj, rk, rl, ri, ridx, &
     &        cg1, cg2, cj, ck, cl, ci, cidx, R
      real(dblprec) rx


      ! All rows of the 2p1h/2p1h block
      if (do2p1h) then
        ridx = wb%numOrbitals + 1
      ! All rows of the 2h1p/2h1p block
      else
        ridx = wb%numOrbitals + cm%num2p1h + 1
      end if

      ! Loop over all rows of (K+C)(k)
      do ri=1, cfgset%numConfigs
        cidx = ridx
        rg1 = cfgset%configs(ri)%g1
        rg2 = cfgset%configs(ri)%g2
        rj  = cfgset%configs(ri)%cj
        rk  = cfgset%configs(ri)%ck
        rl  = cfgset%configs(ri)%cl

        ! Loop over all columns of (K+C)(k)
        do ci=ri, cfgset%numConfigs
          cg1 = cfgset%configs(ci)%g1
          cg2 = cfgset%configs(ci)%g2
          cj  = cfgset%configs(ci)%cj
          ck  = cfgset%configs(ci)%ck
          cl  = cfgset%configs(ci)%cl

          ! Loop over all lattice vectors for the contraction of K+C
          do R=1, cm%numUnitCells

            ! Occupied and virtual orbitals do not couple
            ! Expressions for 2p1h and 2h1p case are formally identical
            rx = 0
            ! Treat the single h in 2p1h or the single p in 2h1p configurations
            if (rl /= cl .or. rg1 == cg1 .or. rg2 == cg2) then
              if (rj == cj .and. rk == ck) then
                rx = rx - ffind(rl, cl)
              end if
            ! Treat the 2p in 2p1h or the 2h in 2h1p configurations
            else ! rl == cl
              if (rk == ck .and. rj /= cj) then
                rx = rx + ffind(rj, cj)
              end if

              if (rk == cj .and. rj /= ck) then
                rx = rx - ffind(rj, ck)
              end if

              if (rj == ck .and. rk /= cj) then
                rx = rx - ffind(rk, cj)
              end if

              if (rj == cj .and. rk /= ck) then
                rx = rx + ffind(rk, ck)
              end if

              if (rj == cj .and. rk == ck) then
                rx = wannierepsilon(cj) + wannierepsilon(ck) - wannierepsilon(cl)
              end if
            end if

            ! If the matrix element does not vanish, write it to disk
            if (rx /= 0) then

              ! Case cj == ck for a column of the 2p1h or 2h1p matrix
              if (cj == ck) then
                call writeentry(am, ridx, cidx, R, rx)
                cidx = cidx + 1
              ! Case cj /= ck for a column of the 2p1h or 2h1p matrix
              else
                call writeentry(am, ridx  , cidx  , R, rx)
                call writeentry(am, ridx+1, cidx+1, R, rx)
                cidx = cidx + 2
              end if

            ! rx == 0
            else

              ! Case cj == ck for a column of the 2p1h or 2h1p matrix
              if (cj == ck) then
                cidx = cidx + 1
              ! Case cj /= ck for a column of the 2p1h or 2h1p matrix
              else
                cidx = cidx + 2
              end if
            end if

          end do
        end do

        if ((cidx-1) /= (cm%num2p1h+wb%numOrbitals) .and. do2p1h) then
          write (*,*) 'COADC_2_3_triple_triple: ridx=', ridx, 'num2p1h+numWannierOrbitals', &
     &                cm%num2p1h+wb%numOrbitals, 'cidx=', cidx-1
          stop '(cidx-1) /= (num2p1h+numWannierOrbitals)'
        end if

        if ((cidx-1) /= (cm%numBands) .and. .not. do2p1h) then
          write (*,*) 'COADC_2_3_triple_triple: ridx=', ridx, 'numADCBands', &
     &                cm%numBands, 'cidx=', cidx-1
          stop '(cidx-1) /= (numADCBands)'
        end if

        if (rj == rk) then
          ridx = ridx + 1
        else
          ridx = ridx + 2
        end if
      end do

      if ((cidx-1) /= (cm%numBands) .and. do2p1h) then
        write (*,*) 'COADC_2_3_triple_triple: ridx=', ridx-1, cm%num2p1h+wb%numOrbitals
      else
        write (*,*) 'COADC_2_3_triple_triple: ridx=', ridx-1, cm%numBands
      end if

#ifdef OLD if
          ! Occupied and virtual orbitals do not couple
          if (do2p1h .and. rl == cl) then
            if (rk == ck) then
              rx = rx + ffind(rj, cj)
            end if

            if (rk == cj) then
              rx = rx - ffind(rj, ck)
            end if

            if (rj == ck) then
              rx = rx - ffind(rk, cj)
            end if

            if (rj == cj) then
              rx = rx + ffind(rk, ck)
            end if

            if (rj == cj .and. rk == ck) then
              rx = wannierepsilon(cj) + wannierepsilon(ck) - wannierepsilon(cl)
            end if
          ! .not. (do2p1h .and. rl == cl)
          else if (.not. do2p1h .and. rj == cj .and. rk == ck) then
            if (rl /= cl) then
              rx = -ffind(rl, cl)
            else
              rx = wannierepsilon(cj) + wannierepsilon(ck) - wannierepsilon(cl)
            end if
          end if
#endif

      end subroutine COADC_2_3_triple_triple


!
! COADC_2_infty_triple_triple --- 2p1h/2p1h or 2h1p/2h1p blocks
!       in CO-ADC, second order for the Coulomb interaction and an
!       exact treatment of the off-diagonal of the Fock matrix
!

      subroutine COADC_2_infty_triple_triple(am, cfgset, do2p1h)
      implicit none

      logical, intent(in) :: do2p1h
      type(tcoadcmatrix), intent(inout) :: am
      type(tconfigurationset), intent(in) :: cfgset
      integer ci, ri, cidx, ridx
      real(dblprec) rx

      ! All rows of the 2p1h/2p1h block
      if (do2p1h) then
        ridx = wb%numOrbitals + 1
      ! All rows of the 2h1p/2h1p block
      else
        ridx = wb%numOrbitals + cm%num2p1h + 1
      end if

      do ri=1, cfgset%numConfigs
        cidx = ridx

        do ci=ri, cfgset%numConfigs
          rx = epsilontensor(cfgset%configs(ri), cfgset%configs(ci), do2p1h)

          ! Case cj == ck for a column of the 2p1h or 2h1p matrix
          if (cfgset%configs(ci)%cj == cfgset%configs(ci)%ck) then

            call writeentry(am, ridx, cidx, R, rx)
            cidx = cidx + 1

          ! Case cj /= ck for a column of the 2p1h or 2h1p matrix
          else

            ! Case cj == ck for a row of the 2p1h or 2h1p matrix
            if (cfgset%configs(ri)%cj == cfgset%configs(ri)%ck) then
              call writeentry(am, ridx  , cidx  , R, rx)
            ! Case cj /= ck for a row of the 2p1h or 2h1p matrix
            else
              call writeentry(am, ridx  , cidx  , R, rx)
              call writeentry(am, ridx+1, cidx+1, R, rx)
            end if
            cidx = cidx + 2

          end if
        end do

        if ((cidx-1) /= (cm%num2p1h+wb%numOrbitals) .and. do2p1h) then
          write (*,*) 'COADC_2_infty_triple_triple: ridx=', ridx, 'num2p1h+numWannierOrbitals', &
     &                cm%num2p1h+wb%numOrbitals, 'cidx=', cidx-1
          stop '(cidx-1) /= (num2p1h+numWannierOrbitals)'
        end if

        if ((cidx-1) /= (cm%numBands) .and. .not. do2p1h) then
          write (*,*) 'COADC_2_infty_triple_triple: ridx=', ridx, 'numADCBands', &
     &                cm%numBands, 'cidx=', cidx-1
          stop '(cidx-1) /= (numADCBands)'
        end if

        if (cfgset%configs(ri)%cj == cfgset%configs(ri)%ck) then
          ridx = ridx + 1
        else
          ridx = ridx + 2
        end if
      end do

      if ((cidx-1) /= (cm%numBands) .and. do2p1h) then
        write (*,*) 'COADC_2_infty_triple_triple: ridx=', ridx-1, cm%num2p1h+wb%numOrbitals
      else
        write (*,*) 'COADC_2_infty_triple_triple: ridx=', ridx-1, cm%numBands
      end if
      end subroutine COADC_2_infty_triple_triple
      end subroutine adcmatrix
