!
! molfock --- diagonalize the molecular Fock matrix
! Christian Buth
! 10.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 10.02.2004 (Took readhf.f code and diagonalize the Fock matrix)
! Version 1.10: 01.03.2004 (Ported to wannier module)
! Version 1.20: 03.03.2004 (Return also the eigenvectors of the Fock matrix upon request)
!
!


!
! Program
!

      subroutine molfock(wann, jobz, eigenvectors, eigenvalues)
      use physconst
      use wannier
      implicit none

      character, intent(in) :: jobz

      integer i, j, info, nonzero

      real(dblprec), intent(out) :: eigenvalues(:), &   ! Eigenvalues of the Fock matrix
     &                              eigenvectors(:,:)   ! Eigenvectors of the Fock matrix
      real(dblprec) largestFij
      real(dblprec), allocatable :: Fock(:), &          ! Upper triangle of the Fock matrix
     &                              work(:)

      type(twannier), intent(in) :: wann


      ! Build the Fock matrix
      write (*,*) 'Diagonalizing the Fock matrix'
      allocate(Fock(wann%numOrbitals * (wann%numOrbitals + 1) / 2), &
     &         work(3 * wann%numOrbitals))

      do j = 1, wann%numOrbitals
        do i = 1, j
          Fock(i + j * (j-1) / 2) = ffind(i,j)
        end do
      end do

#ifndef NDEBUG
      nonzero = 0
      largestFij = 0
      do j = 1, wann%numOrbitals
        do i = 1, j-1
          if (abs(ffind(i,j)) > 1.0D-6) then
            nonzero = nonzero + 1
            if (abs(ffind(i,j)) > largestFij) then
              largestFij = abs(ffind(i,j))
            end if
          end if
        end do
      end do
      write (*,*)
      write (*,*) 'Percentual occupation of Fock matrix ', real(nonzero) / real(wann%numOrbitals &
     &                                                           * (wann%numOrbitals - 1) / 2.0)
      write (*,*) 'Largest off-diagonal element of Fock matrix ', largestFij
#endif

      call dspev(jobz, 'U', wann%numOrbitals, Fock, eigenvalues, eigenvectors, wann%numOrbitals, work, info)
      if (info.ne.0) then
          write (*,*)
          write (*,*) 'dspev returned error while diagonalizing the Fock matrix: info = ', info
          stop 'CO-ADC execution error'
      end if

      write (*,*)
      write (*,*) 'Occupied orbital energies'
      write (*,*) ' # Orbital        epsilon [a.u.]         epsilon [eV]         F_ii'
      do i=1, wann%numOccupied
        write (*,*) i, eigenvalues(i), eigenvalues(i) * HartreetoeV, wannierepsilon(i)
      end do

      write (*,*)
      write (*,*)
      write (*,*) 'Virtual orbital energies'
      write (*,*) ' # Orbital        epsilon [a.u.]         epsilon [eV]         F_ii'
      do i = wann%numOccupied + 1, wann%numOrbitals
        write (*,*) i, eigenvalues(i), eigenvalues(i) * HartreetoeV, wannierepsilon(i)
      end do
      write (*,*)

      deallocate(Fock,work)
      end subroutine molfock
