!
! twointdecay --- determine decay statistics of two-electron
!       integrals in terms of Wannier orbitals
! Christian Buth
! 06.03.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 06.03.2004 (Examine 1h/2p1h-, 1p/2p1h-, 1h/2h1p- and 1p/2h1p-two-electron integrals)
! Version 1.10: 14.03.2004 (Histogram compilation)
!
!


!
! Include files
!

#include "ftypes.h"


!
! Subroutine
!

      subroutine twointdecay
      use wannier
      use adcbands
      implicit none

      integer numHist, latVec, xOrb, oVirt, oOcc, hOrb

      integer, parameter :: numPow = 10

      real rpow10

      real, parameter :: startPow=1, divPow=2

      type twoIntHist
        integer numInt, latticeVec, maxLat(4), maxOrb(4), histData(numPow)
        real(dblprec) maxInt, mean, stdDev
      end type twoIntHist

      type(twoIntHist), allocatable :: intHist(:,:)

      allocate(intHist(4,numUnitCells))
      do numHist=1, 4
        do latVec=1, numUnitCells
          intHist(numHist,latVec)%numInt     = 0
          intHist(numHist,latVec)%latticeVec = 0
          intHist(numHist,latVec)%maxLat     = 0
          intHist(numHist,latVec)%maxOrb     = 0
          intHist(numHist,latVec)%maxInt     = 0
          intHist(numHist,latVec)%mean       = 0
          intHist(numHist,latVec)%stdDev     = 0
          intHist(numHist,latVec)%histData   = 0
        end do
      end do
      write (*,*)
      write (*,*) 'Testing decay of two-electron integrals'

      do xOrb=1, numWannierOrbitals
        do oVirt=1, numWannierVirtual
          do oOcc=1, numWannierOccupied
            do latVec=1, numUnitCells

              ! Decay of 1h/2p1h- and 1p/2p1h-two-electron integrals
              do hOrb=1, oVirt
                call histogramStep(intHist(1,latVec), 1, xOrb, latVec, oVirt, 1, oOcc, latVec, hOrb)
              end do

              ! Decay of 1h/2h1p- and 1p/2h1p-two-electron integrals
  !            do hOrb=1, oOcc
   !             histogram()
    !          end do

            end do
          end do
        end do
      end do


      call histogram(intHist(1,:),numUnitCells)
      write (*,*)
      deallocate(intHist)

        stop 'hist'

      contains

!
!     histogramStep --- create a histogram from the distribution of the two-electron integrals
!

      subroutine histogramStep(intHist, g1, i, g2, j, g3, k, g4, l)
      implicit none

      integer logThresh, g1, i, g2, j, g3, k, g4, l

      real(dblprec) rint, rpow10

      type(twoIntHist) intHist

      rint = gfind(WannierOrbital(g1, i), WannierOrbital(g2, j), &
     &             WannierOrbital(g3, k), WannierOrbital(g4, l))

      if (abs(rint) >= 1.0D-12) then
        intHist%numInt = intHist%numInt + 1
        intHist%mean   = intHist%mean + abs(rint)

        if (abs(rint) >= abs(intHist%maxInt)) then
          intHist%maxInt = rint

          intHist%maxLat(1) = g1
          intHist%maxOrb(1) = i
          intHist%maxLat(2) = g2
          intHist%maxOrb(2) = j
          intHist%maxLat(3) = g3
          intHist%maxOrb(3) = k
          intHist%maxLat(4) = g4
          intHist%maxOrb(4) = l
        end if

        rpow10 = startPow
        do logThresh=1, numPow
          if (abs(rint) >= rpow10) then
            intHist%histData(logThresh) = intHist%histData(logThresh) + 1
            exit
          end if
          rpow10 = rpow10 / divPow
        end do
      end if

      end subroutine histogramStep


!
! histogram --- evaluate and print histogram information
!

      subroutine histogram(intHist, numUnitCells)
      implicit none

      integer numUnitCells, latVec, pow10

      type(twoIntHist) intHist(:)

      do latVec=1, numUnitCells
      intHist(latVec)%mean = intHist(latVec)%mean / intHist(latVec)%numInt
      write (*,'(A1,3I2,A1,2F20.16,8I2)') '(',directlattice(latVec), ')', intHist(latVec)%maxInt, intHist(latVec)%mean, &
     &          intHist(latVec)%maxLat(1), intHist(latVec)%maxOrb(1), intHist(latVec)%maxLat(2), intHist(latVec)%maxOrb(2),   &
     &          intHist(latVec)%maxLat(3), intHist(latVec)%maxOrb(3), intHist(latVec)%maxLat(4), intHist(latVec)%maxOrb(4)
      end do

      write (*,*)
      write (*,*)
      rpow10 = startPow
      do pow10=1, numPow
        write (*,'(ES10.2)',advance='no') rpow10
        rpow10 = rpow10 / divPow

        do latVec=1, numUnitCells
          write (*,'(F10.2)',advance='no') real(intHist(latVec)%histData(pow10)) / intHist(latVec)%numInt * 100
        end do
        write (*,*)
      end do
      end subroutine histogram
      end subroutine twointdecay
