!
! cabplot --- extract information from CO-ADC output for plotting
! Christian Buth
! 05.03.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 05.03.2004 (Crude plotting of band structures, xmgrace output)
! Version 1.10: 12.07.2004 (Support for multipath plotting of band structures,
!       implementation is still somwhat crude)
! Version 1.20: 26.07.2004 (Martin Albrecht cheat mode: adding correlated results
!       to WANNIER's Hartree-Fock bands)
! Version 1.30: 22.02.2005 (Cheat mode using CRYSTAL's Hartree-Fock bands,
!       proper treatment of Fermi energies)
!


!
! Include files
!

#include "assert.h"


!
! Program
!

      program cabplot
      use physconst
      use bandstructure
      implicit none

      type sortidx
        integer numIndices
        integer, pointer :: inew(:), iold(:)
      end type sortidx

      logical plotcoadc, cheat, crystal, resort, cheatcrystal

      character(10) strsetbeg
      character(50) ticklabel(21)               ! Labels for the Brillouinpaths
      character(1024) fileFockBands,       &    ! FOCKALL Hartree-Fock band structure
     &                fileCrystalBands,    &    ! CRYSTAL band structure
     &                fileCOADCBands,      &    ! CO-ADC band structure
     &                fileGrace,           &    ! Output for XMGrace
     &                fileWannierBands(20)      ! Wannier Hartree-Fock band structure
      character(*), parameter :: filegrheader1 = '/home/cbuth/devel/co-adc/trunk/cabplot/grheader1.agr', &
     &          filegrheader2 = '/home/cbuth/devel/co-adc/trunk/cabplot/grheader2.agr', &
     &          filegrset = '/home/cbuth/devel/co-adc/trunk/cabplot/grset.agr'

      integer startOccupied, endVirtual, numADCBands, &
     &        i, &
     &        currPath, numBrillouinPaths
      integer setno, bcno
      integer searchstep, minband, plotbandgap, startquasiband
      integer, parameter :: ufockip       = 50, &
     &                      uadcip        = 51, &
     &                      ufockbands    = 52, &
     &                      ucrystalbands = 53, &
     &                      uadcbands     = 54, &
     &                      ugrace        = 55, &
     &                      ucat          = 56
      integer, allocatable :: plotbands(:), bandsort(:)

      real(dblprec) firstBand, lastBand, polethresh
      real(dblprec) selenergy, selpole, faEfermi, crEfermi, rsEfermi, caEfermi, gpos
      real(dblprec) fabandgap, crbandgap, rsbandgap, cabandgap

      type(tbandsatkpoint) tmpbands
      type(tbandstructure) :: fockallbs, wannierbs, crystalbs, coadcbs, quasibs
      type(tbrillouinpathspec), allocatable :: bpaths(:)
      type(sortidx) fockidx, quasiidx


      namelist /coadcplot/ startOccupied, endVirtual, firstBand, &
     &          lastBand, polethresh, fileFockBands,             &
     &          fileCOADCBands, fileGrace, fileWannierBands,     &
     &          ticklabel, plotcoadc, cheat, startquasiband,     &
     &          crystal, resort, fileCrystalBands, cheatcrystal


      ! Greeting
      write (*,*) 'Crystal orbital algebraic diagrammatic construction', &
     &            'Plotting and data extraction tool'
      write (*,*) 'Copyright (c) 2004-2006 Christian Buth'
      write (*,*)
      write (*,*)

      plotcoadc = .true.
      cheat     = .false.
      crystal   = .false.
      resort    = .false.
      cheatcrystal = .false.

      ! HF chain cc-pVDZ
      startOccupied       =  3
      endVirtual          = 38

      ! LiF crystal ma-vdz
      startOccupied       = 2
      endVirtual          = 23

      ! LiF crystal STO-6G
      startOccupied       = 2
      endVirtual          = 7

      firstBand           = -300
      lastBand            = 300
      polethresh          = 0.6

      ticklabel(1) = 'L'
      ticklabel(2) = '\f{Symbol}G\f{}'
      ticklabel(3) = 'X'
      ticklabel(4) = 'W'
      ticklabel(5) = 'K'
      ticklabel(6) = '\f{Symbol}G\f{}'

      fileFockBands  = 'fockbands.dat'
      fileCOADCBands = 'coadcbands.dat'
      fileGrace      = 'bands.agr'
      fileWannierBands(1) = 'bands_1.dat'
      fileWannierBands(2) = 'bands_2.dat'
      fileWannierBands(3) = 'bands_3.dat'
      fileWannierBands(4) = 'bands_4.dat'
      fileWannierBands(5) = 'bands_5.dat'

      read (*,coadcplot)

      if (cheat .and. cheatcrystal) then
        stop 'Either cheat with Wannier or with Crystal!'
      end if

      ! Read FOCKALL bands
      write (*,*) 'Reading FOCKALL band structure from ', trim(fileFockBands)
      call initbandstructure(fockallbs, ufockbands, fileFockBands)
      call reportbrillouinpathspec(fockallbs)
      write (*,*)
      call fermienergy(fockallbs, faEfermi, fabandgap)
      if (resort) then
        bcno = 6
        fockidx%numIndices = 2
        allocate(fockidx%inew(fockidx%numIndices), fockidx%iold(fockidx%numIndices))
        fockidx%inew(1) = bcno+2 ; fockidx%iold(1) = bcno+3
        fockidx%inew(2) = bcno+3 ; fockidx%iold(2) = bcno+2
!        call resortbands(fockallbs, fockidx, 1, tlatticevector(0.36,0,0), &
!     &                                    tlatticevector(0.5,0,0))
        call resortbands(fockallbs, fockidx, 1, tlatticevector((/0.36,0.0,0.0/)), &
     &                                    tlatticevector((/0.5,0.0,0.0/)))
        fockidx%inew(1) = bcno+3 ; fockidx%iold(1) = bcno+4
        fockidx%inew(2) = bcno+4 ; fockidx%iold(2) = bcno+3
!        call resortbands(fockallbs, fockidx, 1, tlatticevector(0,0,0), &
!     &                                    tlatticevector(0.06,0,0))
        call resortbands(fockallbs, fockidx, 1, tlatticevector((/0.0,0.0,0.0/)), &
     &                                    tlatticevector((/0.06,0.0,0.0/)))
        deallocate(fockidx%inew, fockidx%iold)
      end if
      write (*,*) 'The Fermi energy is ', faEfermi, ' eV', &
     &            ' and the band gap is ', fabandgap, ' eV'
      call bandwidthreport(fockallbs)

      endVirtual = min(endVirtual, fockallbs%numOrbitals + fockallbs%startOccupied)
      write (*,*) 'Writing ', endVirtual - startOccupied + 1, ' Hartree-Fock bands'
      write (*,*)


      ! Read band structure from Wannier
      numBrillouinPaths = fockallbs%numPaths
      write (*,*) 'Reading WANNIER band structure from ', &
     &            (trim(fileWannierBands(i))//'  ', i=1, numBrillouinPaths)
      allocate(bpaths(numBrillouinPaths))
      do currPath=1, numBrillouinPaths
        bpaths(currPath) = fockallbs%paths(currPath)%spec
      end do
      call initbandstructure(wannierbs, ufockbands, fileWannierBands,  &
     &          fockallbs%numPaths, fockallbs%numOccupied + fockallbs%startOccupied + 1,      &
     &          fockallbs%periodicity, bpaths)
      deallocate(bpaths)
      call reportbrillouinpathspec(wannierbs)
      write (*,*)
      call fermienergy(wannierbs, rsEfermi, rsbandgap)
      if (resort) then
        bcno = 6
        fockidx%numIndices = 2
        allocate(fockidx%inew(fockidx%numIndices), fockidx%iold(fockidx%numIndices))
        fockidx%inew(1) = bcno+2 ; fockidx%iold(1) = bcno+3
        fockidx%inew(2) = bcno+3 ; fockidx%iold(2) = bcno+2
!        call resortbands(wannierbs, fockidx, 1, tlatticevector(0.36,0,0), &
!     &                                    tlatticevector(0.5,0,0))
        call resortbands(wannierbs, fockidx, 1, tlatticevector((/0.36,0.0,0.0/)), &
     &                                    tlatticevector((/0.5,0.0,0.0/)))
        fockidx%inew(1) = bcno+3 ; fockidx%iold(1) = bcno+4
        fockidx%inew(2) = bcno+4 ; fockidx%iold(2) = bcno+3
!        call resortbands(wannierbs, fockidx, 1, tlatticevector(0,0,0), &
!     &                                    tlatticevector(0.06,0,0))
        call resortbands(wannierbs, fockidx, 1, tlatticevector((/0.0,0.0,0.0/)), &
     &                                    tlatticevector((/0.06,0.0,0.0/)))
        deallocate(fockidx%inew, fockidx%iold)
      end if
      write (*,*) 'The Fermi energy is ', rsEfermi, ' eV', &
     &            ' and the band gap is ', rsbandgap, ' eV'
      call bandwidthreport(wannierbs)

!      ! Write out Gamma point energies
!      write (*,*) wannierbs%paths(1)%bands(51)%kpoint
!      write (*,'(F15.5)') (wannierbs%paths(1)%bands(51)%energies(i), i=1,14)


      ! Read CRYSTAL bands
      if (crystal) then
        write (*,*) 'Reading CRYSTAL band structure from ', trim(fileCrystalBands)
        call initbandstructure(crystalbs, ucrystalbands, fileCrystalBands)
        call reportbrillouinpathspec(crystalbs)
        write (*,*)
        call fermienergy(crystalbs, crEfermi, crbandgap)
        write (*,*) 'The Fermi energy is ', crEfermi, ' eV', &
     &              ' and the band gap is ', crbandgap, ' eV'
        if (resort) then
          bcno = 6
          fockidx%numIndices = 2
          allocate(fockidx%inew(fockidx%numIndices), fockidx%iold(fockidx%numIndices))
          fockidx%inew(1) = bcno+2 ; fockidx%iold(1) = bcno+3
          fockidx%inew(2) = bcno+3 ; fockidx%iold(2) = bcno+2
!          call resortbands(crystalbs, fockidx, 1, tlatticevector(0.36,0,0), &
!     &                                      tlatticevector(0.5,0,0))
          call resortbands(crystalbs, fockidx, 1, tlatticevector((/0.36,0.0,0.0/)), &
     &                                      tlatticevector((/0.5,0.0,0.0/)))
          fockidx%inew(1) = bcno+3 ; fockidx%iold(1) = bcno+4
          fockidx%inew(2) = bcno+4 ; fockidx%iold(2) = bcno+3
!          call resortbands(crystalbs, fockidx, 1, tlatticevector(0,0,0), &
!     &                                      tlatticevector(0.06,0,0))
          call resortbands(crystalbs, fockidx, 1, tlatticevector((/0.0,0.0,0.0/)), &
     &                                      tlatticevector((/0.06,0.0,0.0/)))
          deallocate(fockidx%inew, fockidx%iold)
        end if
        call bandwidthreport(crystalbs)

        !endVirtual = min(endVirtual, crystalbs%numOrbitals + crystalbs%startOccupied)
        !write (*,*) 'Writing ', endVirtual - startOccupied + 1, ' Hartree-Fock bands'
        write (*,*)

        ! Write out Gamma point energies
!        write (*,*) crystalbs%paths(1)%bands(51)%kpoint
!        write (*,'(F15.5)') (crystalbs%paths(1)%bands(51)%energies(i) - (crEfermi - rsEfermi), i=1,14)
      end if


      ! Read CO-ADC band structure
      if (plotcoadc) then
        call getcoadcbands
      else
        numADCBands = 0
      end if

      call putgraceheader

      ! Fermi energy zero for polymers and surfaces
      ! Testing for HF chains
      if (fockallbs%periodicity /= 3) then
        crEfermi = crEfermi      - rsEfermi
!        stop 'Huh???'
        write (*,*) 'Highly experimental stuff here!!!'
        faEfermi = 0
        rsEfermi = 0
      end if

      ! Transform correlated band structure into a quasiparticle
      ! band structure
      call coadcbands(coadcbs, quasibs, faEfermi)
      if (cheat) then
        write (*,*) 'Applying Martin Albrecht''s cheat with WANNIER bands!'
        call cheatbands(quasibs, wannierbs, rsEfermi, startquasiband)
      else if (cheatcrystal) then
        write (*,*) 'Applying Martin Albrecht''s cheat with CRYSTAL bands!'
        call cheatbands(quasibs, crystalbs, crEfermi + rsEfermi, startquasiband)
      else
        write (*,*) 'FOCKALL bands underlie CO-ADC quasi-particle band structure!'
      end if

      if (resort) then
        bcno = 4
        quasiidx%numIndices = 4
        allocate(quasiidx%inew(quasiidx%numIndices), quasiidx%iold(quasiidx%numIndices))
        quasiidx%inew(1) = bcno+1 ; quasiidx%iold(1) = bcno+1
        quasiidx%inew(2) = bcno+3 ; quasiidx%iold(2) = bcno+2
        quasiidx%inew(3) = bcno+4 ; quasiidx%iold(3) = bcno+3
        quasiidx%inew(4) = bcno+2 ; quasiidx%iold(4) = bcno+4
!        call resortbands(quasibs, quasiidx, 1, tlatticevector(0.5,0,0), &
!     &                                  tlatticevector(0.5,0,0))
        call resortbands(quasibs, quasiidx, 1, tlatticevector((/0.5,0.0,0.0/)), &
     &                                  tlatticevector((/0.5,0.0,0.0/)))
        deallocate(quasiidx%inew, quasiidx%iold)
      end if
      call bandwidthreport(quasibs)

      ! Write FOCKALL bands
      setno = 0
      write (*,*)
      write (*,*) 'Writing FOCKALL bands'
      call putfockbands(ugrace, fockallbs, startOccupied, endVirtual, setno, faEfermi)

      ! Write Wannier bands
      write (*,*)
      write (*,*) 'Writing WANNIER bands'
      call putfockbands(ugrace, wannierbs, startOccupied, endVirtual, setno, rsEfermi)

      ! Read CRYSTAL bands
      if (crystal) then
        write (*,*)
        write (*,*) 'Writing CRYSTAL bands'
        call putfockbands(ugrace, crystalbs, startOccupied, &
     &       min(endVirtual, crystalbs%numOrbitals), setno, crEfermi)
      end if

      ! Write CO-ADC bands
      if (plotcoadc) then
        write (*,*)
        write (*,*) 'Writing CO-ADC bands'
        caEfermi = 0
        call putfockbands(ugrace, quasibs, 1, numADCBands, setno, &
     &       caEfermi, startquasiband)
      end if

      close(unit=ugrace)


      write (*,*)
      write (*,*)
      write (*,*)
      write (*,*) 'Excecution sucessful'
      write (*,*)
      write (*,*) 'Rosarote Brille!'
      write (*,*) '   Immanuel Kant'


!
! Program procedures
!


      contains

!
! concatenatefile --- append the contents of some file to another
!

      subroutine concatenatefile(ucat, ugrace, prepend)
      implicit none

      character(132) grline
      character(*) prepend
      integer ucat, ugrace

      do
        read (ucat,'(A)',end=1000) grline
        write (ugrace,'(A,A)') prepend, trim(grline)
      end do
 1000 continue
      end subroutine concatenatefile


!
! putgraceheader --- Write header and set descriptions in XMGrace format
!

      subroutine putgraceheader
      implicit none

      integer i

      ! Write header for graph file
      write (*,*) 'Write header for graph file'
      open (unit=ugrace, file=fileGrace, status='replace', action='write')
      open (unit=ucat, file=filegrheader1, action='read')
      call concatenatefile(ucat,ugrace,'')
      close (unit=ucat)

      gpos = 0
      write (ugrace,'(A,I2)') '@    xaxis  tick spec ', numBrillouinPaths+1
      do currPath=0, numBrillouinPaths
        write (*,*) 'Tick No. ', currPath+1, ' is ', trim(ticklabel(currPath+1))
        write (ugrace,'(A,I2,A,F8.6)') '@    xaxis  tick major ', currPath, ', ', gpos
        write (ugrace,'(A,I2,A)')      '@    xaxis  ticklabel ', &
     &          currPath, ', "'//trim(ticklabel(currPath+1))//'"'
        if (currPath < numBrillouinPaths) then
          gpos = gpos + fockallbs%paths(currPath+1)%spec%metriclength / fockallbs%metriclength
        end if
      end do

      open (unit=ucat, file=filegrheader2, action='read')
      call concatenatefile(ucat,ugrace,'')
      close (unit=ucat)

      ! Create set description for all sets
      write (*,*) 'Create set description for all sets'
      setno = 0
      open(unit=ucat, file=filegrset, action='read')
      do i=startOccupied, endVirtual
        write (strsetbeg,'(A,I0,A)') '@    s', setno, ' '
        call concatenatefile(ucat,ugrace,trim(strsetbeg)//' ')
        write (ugrace,'(A)') trim(strsetbeg)//' line color 1'
        rewind(unit=ucat)
        setno = setno + 1
      end do
      do i=startOccupied, endVirtual
        write (strsetbeg,'(A,I0,A)') '@    s', setno, ' '
        call concatenatefile(ucat,ugrace,trim(strsetbeg)//' ')
        write (ugrace,'(A)') trim(strsetbeg)//' line color 3'
        rewind(unit=ucat)
        setno = setno + 1
      end do
      if (crystal) then
        do i=startOccupied, min(endVirtual, crystalbs%numOrbitals)
          write (strsetbeg,'(A,I0,A)') '@    s', setno, ' '
          call concatenatefile(ucat,ugrace,trim(strsetbeg)//' ')
          write (ugrace,'(A)') trim(strsetbeg)//' line color 4'
          rewind(unit=ucat)
          setno = setno + 1
        end do
      end if
      do i=1, numADCBands
        write (strsetbeg,'(A,I0,A)') '@    s', setno, ' '
        call concatenatefile(ucat,ugrace,trim(strsetbeg)//' ')
        write (ugrace,'(A)') trim(strsetbeg)//' line color 2'
        rewind(unit=ucat)
        setno = setno + 1
      end do
      close(unit=ucat)
      write (*,*) 'Created ', setno, ' set data records'
      end subroutine putgraceheader


!
! getcoadcbands --- read correlated band structure
!

      subroutine getcoadcbands
      implicit none

      ! Read CO-ADC bands
      call initbandstructure(coadcbs, uadcbands, fileCOADCBands)
      !caEfermi = fermienergy(coadcbs)
      caEfermi = 6
      cabandgap = 0
      write (*,*) 'The Fermi energy is ', caEfermi, ' eV'

      ! Band gap
      plotbandgap=1
      do searchstep=1, coadcbs%paths(1)%bands(1)%numBands
        if (coadcbs%paths(1)%bands(1)%energies(searchstep) > caEfermi .and. &
     &      coadcbs%paths(1)%bands(1)%polestrengths(searchstep) > polethresh) then
          plotbandgap = searchstep - 1
          exit
        end if
      end do
      write (*,*) 'Highest valence band index: ', plotbandgap

      ! Determine the number of CO-ADC bands to write
      allocate(plotbands(coadcbs%paths(1)%bands(1)%numBands))
      write (*,*) 'firstBand, lastBand, polethresh'
      write (*,*) firstBand, lastBand, polethresh
      numADCBands = 0
      do i=1, coadcbs%paths(1)%bands(1)%numBands
        if (coadcbs%paths(1)%bands(1)%energies(i) >= firstBand .and. &
     &      coadcbs%paths(1)%bands(1)%energies(i) <=  lastBand .and. &
     &      coadcbs%paths(1)%bands(1)%polestrengths(i) >= polethresh) then
          numADCBands = numADCBands + 1
          plotbands(numADCBands) = i - plotbandgap
        end if
      end do
      write (*,*) 'numADCBands to write', numADCBands
      end subroutine getcoadcbands


!
! putfockbands -- write XMGrace output of a one-particle (Hartree-Fock)
!       band structure
!

      subroutine putfockbands(ugrace, bs, startOccupied, endVirtual, setno, &
     &          Efermi, startquasiband)
      implicit none

      integer, intent(in) :: startOccupied, endVirtual, ugrace
      integer, intent(inout) :: setno
      integer, intent(in), optional :: startquasiband
      real(dblprec), intent(in) :: Efermi
      type(tbandstructure), intent(in) :: bs
      integer currBand, currPath, currkpoint
      real(dblprec) gsect, gpos, gcurr

      do currBand=startOccupied, endVirtual
        if (present(startquasiband)) then
          write (*,*) 'Writing band No. ', currBand + startquasiband
        else
          write (*,*) 'Writing band No. ', currBand
        end if
        write (ugrace,'(A12,I0)') '@target G0.S', setno
        write (ugrace,'(A)') '@type xy'
        gpos = 0

        do currPath=1, bs%numPaths
          ! Determine fraction of the graph for the path
          gsect = bs%paths(currPath)%spec%metriclength / bs%metriclength &
     &            / (bs%paths(currPath)%spec%numkpoints - 1)

          do currkpoint=1, bs%paths(currPath)%spec%numkpoints
            gcurr = gpos + gsect * (currkpoint - 1)
            write (ugrace,*) gcurr, &
     &        (bs%paths(currPath)%bands(currkpoint)%energies(currBand) - Efermi)
          end do

          gpos = gpos + bs%paths(currPath)%spec%metriclength / bs%metriclength
        end do

        write (ugrace,'(A)') '&'
        setno = setno + 1
      end do
      end subroutine putfockbands


!
! coadcbands --- convert CO-ADC translational symmetry classified
!       energy levels into a quasi-particle band structure. The
!       quasi-particle band structure is given relative
!       to the Fermi energy of the FOCKALL
!       band structure which is assumed to be zero. The
!       Efermi passed to this subroutine merely serves in
!       comparisons to distinguish occupied from
!       virtual quasi-particle bands
!

      subroutine coadcbands(bs, qpbs, Efermi)
      implicit none

      real(dblprec), intent(in) :: Efermi
      type(tbandstructure), intent(in) :: bs
      type(tbandstructure), intent(out) :: qpbs
      integer currBand, currPath, currkpoint, bandgap
      real(dblprec) bandenergy

      ! Initialize quasi-particle band structure
      qpbs%fock            = .true.
      qpbs%numPaths        = bs%numPaths
      qpbs%totalnumkpoints = bs%totalnumkpoints
      qpbs%numOccupied     = bs%numOccupied - bs%startOccupied + 1
      qpbs%numOrbitals     = numADCBands
      qpbs%startOccupied   = 1
      qpbs%periodicity     = bs%periodicity
      qpbs%spacegroup      = bs%spacegroup
      qpbs%metriclength    = bs%metriclength
      qpbs%coordlength     = bs%coordlength
      allocate(qpbs%paths(qpbs%numPaths))

      do currPath=1, qpbs%numPaths
        qpbs%paths(currPath)%spec = bs%paths(currPath)%spec
        allocate(qpbs%paths(currPath)%bands(qpbs%paths(currPath)%spec%numkpoints))

        do currkpoint=1, qpbs%paths(currPath)%spec%numkpoints
          qpbs%paths(currPath)%bands(currkpoint)%numBands = numADCBands
          qpbs%paths(currPath)%bands(currkpoint)%kpoint = &
     &             bs%paths(currPath)%bands(currkpoint)%kpoint
          allocate(qpbs%paths(currPath)%bands(currkpoint)%energies(numADCBands))
          nullify(qpbs%paths(currPath)%bands(currkpoint)%polestrengths)
        end do
      end do

      ! Extract quasi-particle bands from CO-ADC data
      do currBand=1, numADCBands
        minband   = plotbands(currBand) + plotbandgap
        selenergy = bs%paths(1)%bands(1)%energies(minband)
        selpole   = bs%paths(1)%bands(1)%polestrengths(minband)

        write (*,*) 'Form quasi-particle band No. ', currBand, selenergy, selpole

        do currPath=1, bs%numPaths
          newkpoint: do currkpoint=1, bs%paths(currPath)%spec%numkpoints

            ! Search band gap
            do searchstep=1, bs%paths(currPath)%bands(currkpoint)%numBands
              if (bs%paths(currPath)%bands(currkpoint)%energies(searchstep) > Efermi   &
     &            .and. bs%paths(currPath)%bands(currkpoint)%polestrengths(searchstep) &
     &            > polethresh) then
                bandgap = searchstep - 1
                exit
              end if
            end do

            ! Find bands
            minband = 0
            do i=1, bs%paths(currPath)%bands(currkpoint)%numBands
              if (bs%paths(currPath)%bands(currkpoint)%energies(i) >= firstBand .and. &
     &            bs%paths(currPath)%bands(currkpoint)%energies(i) <=  lastBand .and. &
     &            bs%paths(currPath)%bands(currkpoint)%polestrengths(i) >= polethresh) then
                minband = minband + 1
                if (minband == currBand) then
                  minband = i
                  go to 14
                end if
              end if
            end do

            cycle newkpoint

            ! The Fermi energy of the FOCKALL band structure is
            ! assumed to be zero. The quasi-particle band structure
            ! is shifted appropriately to abide by this convention.
 14         continue
            bandenergy = bs%paths(currPath)%bands(currkpoint)%energies(minband) &
     &                          - faEfermi
            qpbs%paths(currPath)%bands(currkpoint)%energies(currBand) = bandenergy

          end do newkpoint
        end do
      end do
      end subroutine coadcbands


!
! cheatbands --- add the correlation contribution of the band structure to the
!       CRYSTAL or WANNIER bands which agree much better with the crystal bands
!       than those obtained by diagonalizing the FOCKALL matrix
!

      subroutine cheatbands(qpbs, cheatbs, Efermi, startquasiband)
      implicit none

      integer, intent(in) :: startquasiband
      real(dblprec), intent(in) :: Efermi
      type(tbandstructure), intent(in) :: cheatbs
      type(tbandstructure), intent(inout) :: qpbs
      integer currBand, currPath, currkpoint, ikpct, ikpfa
      real(dblprec) bandenergy, bct, bfa
      type(tlatticevector) kp

/*************************************************************************************
      ! Extract quasi-particle bands from CO-ADC data
      do currBand=1, qpbs%numOrbitals
        do currPath=1, qpbs%numPaths
          do currkpoint=1, qpbs%paths(currPath)%spec%numkpoints

            bandenergy = qpbs%paths(currPath)%bands(currkpoint)% &
     &                          energies(currBand) - Efermi
            kp1 = qpbs%paths(currPath)%bands(currkpoint)%kpoint

            ! Search the CO-ADC k-point in the set of k-points given
            ! in the Hartree-Fock band structure for cheating
            !write (*,*) 'Searching ', kp1, currBand + startquasiband, ' Path ', currPath
            !write (*,*) 'currBand = ', currBand, ', startquasiband = ', startquasiband
            do i=1, cheatbs%paths(currPath)%spec%numkpoints
              kp2 = cheatbs%paths(currPath)%bands(i)%kpoint
!              kp2%v = matmul(mt%m, kp2%v)   ! Transformation to conventional unit cell
!              if (sqrt((kp1%a - kp2%a)**2 + (kp1%b - kp2%b)**2 &
!     &            + (kp1%c - kp2%c)**2) < 1.0D-6) then
!              if (sqrt((abs(kp1%v(1)) - abs(kp2%v(1)))**2 &
!     &                + (abs(kp1%v(2)) - abs(kp2%v(2)))**2 &
!     &               + (abs(kp1%v(3)) - abs(kp2%v(3)))**2) < 1.0D-6) then
              if (sqrt((kp1%v(1) - kp2%v(1))**2 + (kp1%v(2) - kp2%v(2))**2 &
     &               + (kp1%v(3) - kp2%v(3))**2) < 1.0D-6) then
!                write (*,*) 'Found ', kp1, 'Band ', currBand &
!     &                  + startquasiband, ' Path ', currPath
                bec = bandenergy + cheatbs%paths(currPath)%        &
     &                bands(i)%energies(currBand + startquasiband)
                bef = fockallbs%paths(currPath)%bands(i)%energies  &
     &                (currBand + startquasiband)
                bandenergy = (bec - Efermi) - (bef - faEfermi)
                go to 13
              end if
            end do

            ! k-point from the CO-ADC band structure not found in the
            ! Hartree-Fock band structure
            write (*,*) 'k-point ', kp1, ' in path ', currPath, &
     &                  ' not found for cheating!'
            stop 'cabplot: k-point not found for cheating!'

            ! Store modified energy
 13         continue
            qpbs%paths(currPath)%bands(currkpoint)%energies(currBand) = bandenergy
*************************************************************************************/

      ! Extract quasi-particle bands from CO-ADC data
      do currBand=1, qpbs%numOrbitals
        do currPath=1, qpbs%numPaths
          do currkpoint=1, qpbs%paths(currPath)%spec%numkpoints
            ! Search the CO-ADC k-point in the set of k-points given
            ! in the Hartree-Fock band structure for cheating
            kp = qpbs%paths(currPath)%bands(currkpoint)%kpoint
            ikpct = findkpoint(cheatbs  %paths(currPath), kp)
            ikpfa = findkpoint(fockallbs%paths(currPath), kp)

            ! Shift the quasi-particle band appropriately
            bandenergy = qpbs%paths(currPath)%bands(currkpoint)%energies(currBand)
            bct = cheatbs  %paths(currPath)%bands(ikpct)%energies &
     &            (currBand + startquasiband)
            bfa = fockallbs%paths(currPath)%bands(ikpfa)%energies &
     &            (currBand + startquasiband)
            bandenergy = bandenergy + (bct - Efermi) - (bfa - faEfermi)
            qpbs%paths(currPath)%bands(currkpoint)%energies(currBand) = bandenergy
          end do
        end do
      end do
      end subroutine cheatbands


!
! bandwidthreport --- report band widths
!

      subroutine bandwidthreport(bs)
      implicit none

      type(tbandstructure), intent(in) :: bs
      integer i
      real(dblprec) bw
      type(tbspoint) :: bspmin, bspmax

      write (*,*)
      write (*,*) 'Analysis of bands:'
      do i=1, bs%numOrbitals
        bw = bandwidth(bs, i, bspmin, bspmax)
        write (*,'(A,I3,A,3(F10.5,A))') ' Band No. ', i, ':  width of ', bw, &
     &          ' eV, minimum of ', &
     &           bs%paths(bspmin%pathno)%bands(bspmin%kpointno)%energies(i), &
     &          ' eV, maximum of ', &
     &           bs%paths(bspmax%pathno)%bands(bspmax%kpointno)%energies(i), &
     &          ' eV'

!, &
!     &          'eV, minimum at k-point ', &
!     &           bs%paths(bspmin%pathno)%bands(bspmin%kpointno)%kpoint, &
!     &          ', maximum at k-point ', &
!     &           bs%paths(bspmax%pathno)%bands(bspmax%kpointno)%kpoint
      end do
      write (*,*)
      end subroutine bandwidthreport


!
! resortbands --- resort bands to match symmetry requirements
!

      subroutine resortbands(bs, indices, currPath, kbeg, kend)
      implicit none

      integer, intent(in) :: currPath
      type(sortidx), intent(in) :: indices
      type(tlatticevector), intent(in) :: kbeg, kend
      type(tbandstructure), intent(in) :: bs
      integer currkpoint, ikbeg, ikend, i, j
      real(dblprec) rkbeg, rkend
      type(tlatticevector) kp

      ! Search index of beginning and ending k-points
      ikbeg = huge(ikbeg)
      ikend = huge(ikend)
      do currkpoint=1, bs%paths(currPath)%spec%numkpoints
        kp = bs%paths(currPath)%bands(currkpoint)%kpoint
!        rkbeg = sqrt((kp%a - kbeg%a)**2 + (kp%b - kbeg%b)**2 + (kp%c - kbeg%c)**2)
!        rkend = sqrt((kp%a - kend%a)**2 + (kp%b - kend%b)**2 + (kp%c - kend%c)**2)
        rkbeg = sqrt((kp%v(1) - kbeg%v(1))**2 + (kp%v(2) - kbeg%v(2))**2 &
     &             + (kp%v(3) - kbeg%v(3))**2)
        rkend = sqrt((kp%v(1) - kend%v(1))**2 + (kp%v(2) - kend%v(2))**2 &
     &             + (kp%v(3) - kend%v(3))**2)
        if (rkbeg < 1.0D-6) then
          ikbeg = currkpoint
        end if
        if (rkend < 1.0D-6) then
          ikend = currkpoint
        end if
      end do

      if (ikbeg == huge(ikbeg) .or. ikend == huge(ikend)) then
        stop 'resortbands: kpoint not found!'
      end if

      if (ikbeg > ikend) then
        i = ikbeg
        ikbeg = ikend
        ikend = i
      end if

      write (*,*) 'Resorting bands between ', ikbeg, ' and ', ikend

      ! Specify resorting
      tmpbands%numBands = bs%paths(currPath)%bands(ikbeg)%numBands
      allocate(tmpbands%energies(tmpbands%numBands),      &
     &         tmpbands%polestrengths(tmpbands%numBands), &
     &         bandsort(tmpbands%numBands))

      do i=1, bs%paths(currPath)%bands(ikbeg)%numBands
        bandsort(i) = i
      end do

      do i=1, indices%numIndices
        bandsort(indices%inew(i)) = indices%iold(i)
      end do

!      write (*,*) 'bcno, bs%paths(currPath)%bands(ikbeg)%numBands'
!      write (*,*) bcno, bs%paths(currPath)%bands(ikbeg)%numBands
!      bandsort(bcno+1) = bcno+1
!      bandsort(bcno+3) = bcno+2
!      bandsort(bcno+4) = bcno+3
!      bandsort(bcno+2) = bcno+4

      ! Resort bands
      do currkpoint = ikbeg, ikend
        do i=1, bs%paths(currPath)%bands(currkpoint)%numBands
          j = bandsort(i)
          tmpbands%energies(j)      = bs%paths(currPath)%bands(currkpoint)%energies(i)
!          tmpbands%polestrengths(j) = bs%paths(currPath)%bands(currkpoint)%polestrengths(i)
          if (i /= j) then
            write (*,*) 'Resorting band no ', i, ' to ', j, ' energy ', tmpbands%energies(j)
          end if
        end do

        bs%paths(currPath)%bands(currkpoint)%energies(:) = tmpbands%energies(:)
!        bs%paths(currPath)%bands(currkpoint)%polestrengths(:) = tmpbands%polestrengths(:)
      end do
      deallocate(tmpbands%energies, tmpbands%polestrengths, bandsort)
      end subroutine resortbands
      end program cabplot
