!
! fockdiag --- analyse the Fock matrix
! Christian Buth
! 06.09.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.30: 20.06.2004 (Separated diagonalization code, matrix written to disk)
!


!
! Program
!

      program fockdiag
      use physconst
      use wannier
      use adcbands
      implicit none

      character(1024) fileFockip,     &                 ! Negative of Hartree-Fock orbital energies
     &                fileFockBands,  &                 ! FOCKALL Hartree-Fock band structure
     &                fileFockAll,    &                 ! Wannier direct space Fock matrices
     &                fileFockTrans,  &                 ! Wannier Fock matrix of the integral transformed unit cells
     &                fileVBucket,    &                 ! Wannier two-electron bucket file
     &                fileVijkl,      &                 ! Wannier two-electron integrals
     &                fileVIndex                        ! Wannier two-electron index file

      integer numUnitCells, startOccupied, endVirtual, periodicity, numBrillouinPaths, &
     &        orbs1(5), orbs2(5), i, j, R
      integer, parameter :: ffockip    = 50, &
     &                      fadcip     = 51, &
     &                      ffockbands = 52, &
     &                      fadcbands  = 53

      real(dblprec) poke

      type(twannier) wannbands
      type(tcoadc) coadcmat

      namelist /coadcmatrix/ numUnitCells, startOccupied, endVirtual, periodicity, &
     &          numBrillouinPaths, fileFockip, fileFockBands,        &
     &          fileFockAll, fileFockTrans, fileVBucket, fileVijkl,  &
     &          fileVIndex


      ! Greeting
      write (*,*) 'Crystal orbital algebraic diagrammatic constuction'
      write (*,*) '(c) 2004 Christian Buth and Martin Albrecht'
      write (*,*)
      write (*,*)

      numUnitCells  = 1
      startOccupied =  1
      endVirtual    = -1
      periodicity   = 0
      periodicity   = 1
      numBrillouinPaths = 1

      fileFockip     = 'fockip.dat'
      fileFockBands  = 'fockbands.dat'
      fileFockAll    = 'FOCKALL0i1.DAT'

      !read (*,coadcmatrix)

      ! Read FOCKALL
      wannbands%periodicity = periodicity
      call fockall(wannbands, fileFockAll)
      call fockparam(wannbands)

      if (endVirtual < 0) then
        endVirtual = wannbands%numOrbitals
      end if

      ! Carry out CO-ADC calculation
      select case (wannbands%periodicity)
        case (0)
          write (*,*) 'The system is a molecule'
        case (1)
          write (*,*) 'The system is a polymer'
        case (2)
          write (*,*) 'The system is a surface'
        case (3)
          write (*,*) 'The system is a crystal'
      end select

        call writefockall

      ! Fiddle around in the FOCKALL for symmetry adaption
      orbs1(1) = 1
      orbs1(2) = 4
      orbs1(3) = 6
      orbs1(4) = 7
      orbs1(5) = 9

      orbs2(1) = 2
      orbs2(2) = 3
      orbs2(3) = 5
      orbs2(4) = 8
      orbs2(5) = 10

      do R=2, 8, 2
        do i=1, wannbands%numOrbitals
          do j=1, wannbands%numOrbitals

            poke = (fallfind(i,R,j) + fallfind(j,R+1,i)) / 2D0
            call fallpoke(i, R  , j, poke)
            call fallpoke(j, R+1, i, poke)

          end do
        end do
      end do

      do R=1, 9
        do i=1, 37, 2
          j=i+1
        poke = (abs(fallfind(i,R,i)) + abs(fallfind(j,R,j))) / 2D0
        call fallpoke(i, R, i, sign(poke, fallfind(i,R,i)))
        call fallpoke(j, R, j, sign(poke, fallfind(j,R,j)))
        end do
      end do



        call writefockall




      if (wannbands%periodicity == 0) then
        call ipea
      else
        call bandstruct
      end if


      write (*,*)
      write (*,*)
      write (*,*)
      write (*,*) 'Excecution sucessful'
      write (*,*)
      write (*,*) 'Da steh ich nun, ich armer Tor!'
      write (*,*) 'Und bin so klug als wie zuvor;'
      write (*,*) '                Goethe, Faust I'


      contains

!
! ipea --- calculate the Hartree-Fock and ADC ionization potentials and electron affinities
!

      subroutine ipea
      implicit none

      logical exactoffdiag, fockpt3

      integer i

      real(dblprec), allocatable :: fockeigenvectors(:,:), &             ! Eigenvectors of the Fock matrix
     &                              fockeigenvalues(:)                   ! Eigenvalues of the Fock matrix

      ! The Fock matrix of the molecule
      allocate(fockeigenvectors(wannbands%numOrbitals,wannbands%numOrbitals), &
     &         fockeigenvalues(wannbands%numOrbitals))
      write (*,*) 'Calculating the Hartree-Fock ionization potentials and electron affinities...'
      open(unit=ffockip,file=fileFockip,status='replace',action='write')
      !!!!!!!!call molfock(wannbands, 'V', fockeigenvectors,fockeigenvalues)
      do i=startOccupied, endVirtual
        write (ffockip,'(2F15.6)') -fockeigenvalues(i) * HartreetoeV, 1.0
      end do
      write (ffockip,*)
      close(unit=ffockip)
      deallocate(fockeigenvectors,fockeigenvalues)
      end subroutine ipea


!
! bandstruct --- calculate the Hartree-Fock and CO-ADC band structures
!

      subroutine bandstruct
      use bandstructure
      implicit none

      logical exactoffdiag, fockpt3
      integer currPath, currkpoint, uplot, i
      real(dblprec), allocatable :: eigenvalues(:)                      ! Eigenvalues of the Fock matrix
      real(dblprec), allocatable :: dummy(:,:)
      type(tlatticevector) kpoint
      type(tfbands) fbands

      ! Get paths though the Brillouin zone
      allocate(fbands%paths(numBrillouinPaths))
      fbands%fock = .true.
      fbands%numPaths = numBrillouinPaths
      fbands%startOccupied = 1
      fbands%numOccupied = wannbands%numOccupied
      fbands%numOrbitals = wannbands%numOrbitals
      fbands%periodicity = wannbands%periodicity
      do currPath=1, numBrillouinPaths
      !!!!!  call readbrillouinpathspec(fbands%paths(currPath), fstdin, wannbands%periodicity, 0)
        fbands%paths(currPath)%beg = tlatticevector(0,0,0)
        fbands%paths(currPath)%end = tlatticevector(0.5,0,0)
        fbands%paths(currPath)%inc = tlatticevector(0.01,0,0)
        call initbrillouinpathspec(fbands%paths(currPath), wannbands%periodicity, 0)
      end do
      call reportbrillouinpathspec(fbands%paths, numBrillouinPaths, wannbands%periodicity)

      ! Hartree-Fock band structure
      allocate(eigenvalues(wannbands%numOrbitals))

      write (*,*) 'Diagonalizing the Fock matrix...'
      call openbands(fbands, ffockbands, fileFockBands, 'write')
        uplot = 199
      open(unit=uplot, file='fockplot.dat', status='replace', action='write')

      do currPath=1, numBrillouinPaths
        call putbrillouinpath(fbands, currPath)
        kpoint = fbands%paths(currPath)%beg
        do currkpoint=1, fbands%paths(currPath)%numkpoints
          call wannierfock(wannbands, kpoint, eigenvalues)
          eigenvalues = eigenvalues * HartreetoeV
          call putbandrecord(fbands, kpoint, wannbands%numOrbitals, eigenvalues)

          write (uplot,'(30(F0.4,A))') kpoint%a, ' ', kpoint%b, ' ', kpoint%c, ' ', &
     &          (eigenvalues(i), ' ', i=5, 12)

          kpoint%a = kpoint%a + fbands%paths(currPath)%inc%a
          kpoint%b = kpoint%b + fbands%paths(currPath)%inc%b
          kpoint%c = kpoint%c + fbands%paths(currPath)%inc%c
        end do
      end do

      close(unit=uplot)
      call closebands(fbands)
      write (*,*) ' Hartree-Fock bands are in ',trim(fileFockBands)
      write (*,*)

      deallocate(eigenvalues)
      end subroutine bandstruct


!
! writefockall --- write Fock matrices
!

      subroutine writefockall
      implicit none

      ! The Fockmatrix
      write (*,*)
      write (*,*)
      write (*,*)

      do R=1, 9
        write (*,*) 'diag(F(', R, ')):'
        write (*,'(50(F8.4,A))') (fallfind(i,R,i), ' ', i=1, wannbands%numOrbitals)
        write (*,*)
        write (*,*)
        write (*,*)
      end do


      do R=1, 9
        write (*,*) 'F(', R, '):'
        do i=1, wannbands%numOrbitals
          write (*,'(50(F8.4,A))') (fallfind(i,R,j), ' ', j=1, wannbands%numOrbitals)
        end do
        write (*,*)
        write (*,*)
        write (*,*)
      end do

      write (*,*)
      write (*,*)
      write (*,*)
      end subroutine writefockall
      end program fockdiag
