!
! geommolpro --- convert Wannier98 geometry output into MOLPRO format
! Christian Buth
! 18.04.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 18.09.2004 (Read in Wannier cluster geometry and
!       write out MOLPRO xyz geometry)
!


      program geommolpro
      use fortrantl
      implicit none

      character(4) element
      character(132) line
      character(*), parameter :: symopstr = ' no. of symmetry operations.:'

      integer iatom, numAtoms, cardinal
      real(dblprec), parameter :: BohrtoAngstroem = 0.5291772083D0
      real(dblprec) posx, posy, posz, nuccharge, nucmass

      do
        read (*,'(A29)') line
        if (trim(line) == symopstr) then
          read (*,'(A15,I)') line, numAtoms
          read (*,*)
          read (*,*)
          read (*,*)
          write (*,'(A)') 'geomtyp=xyz'
          write (*,*)
          write (*,'(A)') 'geometry={'
          write (*,'(A)') 'nosym, noorient'
          write (*,*) numAtoms, '               ! Number of atoms'
          write (*,'(A)') 'Coordinates of atoms in the Wannier98 cluster'

          do iatom=1, numAtoms
            read (*,'(I4,A4,5F15.8)') cardinal, element, posx, posy, posz, nuccharge, nucmass
            write (*,'(A2,3(A,ES15.8))') adjustl(element), ' ,', posx * BohrtoAngstroem, &
     &            ',', posy  * BohrtoAngstroem, ',', posz * BohrtoAngstroem
          end do

          write (*,'(A)') '}'
          stop
        end if
      end do

 100  stop 'Error: not a Wannier98 output'
      end program geommolpro
