!
! hblanc --- band Lanczos for complex Hermitian matrices
! Hans-Dieter Meyer
!
!
! Copyright (c) 2003 Hans-Dieter Meyer
! Copyright (c) 2003 Thomas Sommerfeld
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
c     ---------------------------------------------------------
c     read the restartfile
c     --------------------
      if (restart .ne. 0) then
         write (6,'(4x,A)') 'Restart from file'
         inquire(file='RESTART', exist=da)
         if (.not. da) then
            write(6,*) 'RESTART File is missing.'
            stop
         endif
         open (ifile,file='RESTART',access='SEQUENTIAL',
     $        form='UNFORMATTED')
         read(ifile) jend, j
         if (j .ne. ndim) then
            write(6,*) 'Inconsistancy:'
            write(6,*) 'Ndim(Inputfiles)  = ', ndim
            write(6,*) 'Ndim(Restartfile) = ', j
            stop
         endif
         write(6,'(6x,A,I5)') 'Iterations done so far  ', jend
         write(6,'(6x,A,I5)') 'Iterations to be done   ', niter-jend
         if (jend .ge. niter) then
            stop 'Nothing to do'
         endif
         read(ifile) ((t(i,j), i=1, np + 1),  j=1, jend)
         read(ifile) ((p(i,j,1), i=1, ndim), j=1, np),
     $            ((p(i,j,2), i=1, ndim), j=1, np)
         read(ifile) ncartsv
         read(ifile) (icartsv(i), i=1, ncartsv)
         close (ifile)
         jstart = jend + 1
      endif
c     ----------------------------
c     print Iterations and runtype
c     ----------------------------
      write (6,'(4x,A,I5,A,I5)') 'Iterations from ',jstart,' to ',niter
      if (neigenv .gt. 0) then
         write(6,'(4X,A,I4,A)') 'This is a second run computing ',
     $        neigenv,' eigenvectors'
      endif
      write(6,*)
c     ---------------------------------------------------------------
c     prepare for eigenvector run
c     ---------------------------
      if (neigenv .gt. 0) then
c     --------------------------------------
c     read the ZMAT file to z(niter,neigenv)
c     this file is created by banddia after
c     a first lanczos requiring eigenvectors
c     --------------------------------------
        inquire(file='ZMAT', exist=da)
         if (.not. da) then
            write(6,*) '**** The File ZMAT does not exist.'
            stop
         endif
         open (ifile,file='ZMAT',access='SEQUENTIAL',
     $        form='UNFORMATTED')
         read(ifile) i, j
         if (i .ne. niter .or. j .ne. neigenv) then
            write(6,*) '***************   WARNING  *****************'
            write(6,*) ' ZMAT file _may_ be corrupted.'
            write(6,*) ' niter, neigenv (Args) = ', niter, neigenv
            write(6,*) ' zdim,  neigenv (File) = ', i, j
            write(6,*) ' (niter must be too small in a restart set up)'
            write(6,*)
            if (j .lt. neigenv .or. i .lt. niter) then
               stop
            endif
         endif
         do j = 1, neigenv
            read(ifile) (z(i,j), i = 1, niter)
         enddo
         close (ifile)
c     -------------------------------
c     initialize or read eigenvectors
c     -------------------------------
         if (restart .eq. 0) then
            do i = 1, neigenv
               do j = 1, ndim
                  ev(j,i) = (0.0d0, 0.0d0)
               enddo
            enddo
            do j = 1, np
               jiter = jstart - 1 + j
               do i = 1, neigenv
                  do jko = 1, ndim
                     ev(jko,i) = ev(jko,i)+p(jko,j,2)*z(jiter,i)
                  enddo
               enddo
            enddo
c
         else
c     ------------------------------------------------------
c     restart = 1
c     read eigenvectors from the EIGENV file of the last run
c     ------------------------------------------------------
            inquire(file='EIGENV', exist=da)
            if (.not. da) then
               write(6,*) 'EIGENV File is missing.'
               stop
            endif
            open (ifile,file='EIGENV',access='SEQUENTIAL',form
     $           ='UNFORMATTED')
            read(ifile) i, j
            if (i .ne. neigenv .or. j .ne. ndim) then
               write(6,*) 'EIGENV file may be corrupted.'
               write(6,*) 'ndim, neigenv (Args) = ', ndim, neigenv
               write(6,*) 'ndim, neigenv (File) = ', j, i
               stop
            endif
            read(ifile) i
            if (i .ne. 2) then
               write(6,*) 'itype = ',i,' ?????'
               stop
            endif
            do j = 1, neigenv
               read(ifile) (ev(i,j), i = 1, ndim)
            enddo
            close(ifile)
c     ------------------------------------------
c     contributions of the restart vectors which
c     have NOT been added in the previous run!
c     ------------------------------------------
            do j = 1, np
               jiter = jstart - 1 + j
               do i = 1, neigenv
                  do jko = 1, ndim
                     ev(jko,i) = ev(jko,i) + p(jko,j,2) * z(jiter,i)
                  enddo
               enddo
            enddo
         endif
      endif
c     ============================================================
c     end of eigenvector preparations
c
