!
! lanczos --- Lanczos diagonalization routines
! Christian Buth
! 05.08.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
! Version 1.00: 05.08.2004 (Explicit interfaces for all FORTRAN procedures)
!


!
! Module
!

      module lanczos
      use fortrantl
      implicit none


!
! Parameters
!

      integer, parameter :: piterdone        = 1, &     ! Band Lanczos iteration done
     &                      pmatblk          = 2, &     ! Carry out block vector times matrix operation
     &                      planczosfinished = 3        ! Everything done


!
! Types
!

      type thlanczos
        integer ndim,  &
     &          np,    &
     &          niter, &
     &          jstart, &
     &          jend,   &
     &          jj, in, io, jn
        complex(dblprec), pointer :: lanvecs(:,:,:), &            ! Block of Lanczos vectors
     &                               Tmat(:,:)                  ! Bandmatrix
      end type thlanczos


!
! Interfaces
!


      interface
      end interface


      interface initlanczos
        module procedure initlanczos_h
      end interface initlanczos


!
! Module procedures
!

      contains

!
! initlanczos_h
!

      subroutine initlanczos_h(lan, ndim, np, niter)
      implicit none

      integer, intent(in) :: ndim, np, niter
      type(thlanczos), intent(inout) :: lan
      integer i

      ! Initialize Lanczos
      lan%ndim    = ndim
      lan%np      = np
      lan%niter   = niter
      lan%jstart  = 1
      lan%jj      = lan%jstart

      if (present(jstart)) then
      else
      end if

      allocate(lan%lanvecs(ndim, np, 3), lan%Tmat(np+1, niter))

      ! Generate canonical start vectors
      lan%lanvecs = 0
      do i=1, np
        lan%lanvecs(i, i, 2) = (1.0, 0.0)
      end do

      end subroutine initlanczos_h
      end module lanczos
