!
! physconst --- Physical constants and unit conversion factors
! Christian Buth
! 27 Jun 2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
! 2002 CODATA recommended values
! Taken from
!       http://physics.nist.gov/cuu/Constants
! which in turn is based on the article
!       Reviews of Modern Physics 77, 1--107 (2005).
!
!
! Version 1.00: 27 Jun 2004 (BohrToAngstroem and HartreeToeV)
! Version 1.10: 01 Aug 2006 (AngstroemToBohr, eVToHartree, RydbergToeV,
!       intensity_AUtoSI, inverse_fine_structure_constant, fine_structure_constant)
!


!
! Module data
!

      module physconst
      use fortrantl
      implicit none


!
! Parameters
!

      real(dblprec), parameter ::                                                       &
!
! Length
!
     &          BohrToAngstroem                 = 0.5291772083D0,                       &
     &          AngstroemToBohr                 = 1.0D0 / BohrToAngstroem,              &
     &          BohrToCentimeter                = BohrToAngstroem * 1.0D-8,             &
     &          CentimeterToBohr                = 1.0D0 / BohrToCentimeter,             &
     &          MicrometerToBohr                = AngstroemToBohr * 1.0D4,              &
     &          BohrToMicrometer                = 1.0D0 / MicrometerToBohr,             &
!
! Energy
!
     &          HartreeToeV                     = 27.2113845D0,                         &
     &          eVToHartree                     = 1.0D0 / HartreeToeV,                  &
     &          RydbergToeV                     = 13.6056923D0,                         &
     &          JouleToHartree                  = 2.29371257D17,                        &
     &          NanojouleToHartree              = JouleToHartree / 1.0D9,               &
     &          HartreeToJoule                  = 4.35974417D-18,                       &
     &          JouleToElectronvolt             = 6.24150965D18,                        &
     &          KelvinToHartree                 = 3.1668153D-6,                         &
     &          HartreeToKelvin                 = 1.0D0 / KelvinToHartree,              &
     &          invCentimeterToHartree          = 4.556335252760D-6,                    &
      ! 1 E_h / h; not the factor of 2 pi difference to 1 / t0ToSecond
     &          HartreeToHertz                  = 6.579683920722D15,                    &
!
! Misc
!
      ! The intensity in SI is W / cm^2 and in AU it is E_h / t_0 / a_0^2
     &          intensity_AUtoSI                = 6.436409085121321D15,                 &
     &          intensity_SItoAU                = 1.0D0 / intensity_AUtoSI,             &
      ! SI: alpha = e^2 / (4 PI epsilon_0 hbar c); unit:
      ! AU: alpha = 1 / c; unit: t_0 / a_0
     &          fine_structure_constant         = 7.297352568D-3,                       &
     &          inverse_fine_structure_constant = 137.03599911,                         &
      ! 1 barn (b) = 10^-24 cm^2 = 10^-28 m^2 = 10^-8 Angstroem^2 = 100 fm^2
     &          cross_section_AUtoBarn          = 1.0D8 * BohrToAngstroem**2,           &
      ! 1 t_0 = hbar / E_h
!
! Time
!
      ! t0 denotes the atomic unit of time
      ! 1 hbar / E_h
     &          t0ToSecond                      = 2.418884326505D-17,                   &
     &          t0ToPicosecond                  = t0ToSecond * 1.0D12,                  &
     &          t0ToFemtosecond                 = t0ToSecond * 1.0D15,                  &
     &          SecondTot0                      = 1.0D0 / t0ToSecond,                   &
     &          PicosecondTot0                  = 1.0D0 / t0ToPicosecond,               &
     &          FemtosecondTot0                 = 1.0D0 / t0ToFemtosecond,              &
!
! Fundamental constants
!
      ! Electron mass, 1 kg
     &          ElectronMass                    = 9.10938215D-31,                       &
      ! Elementary charge, 1 C
     &          ElementaryCharge                = 1.602176487D-19,                      &
      ! speed of light in vacuum, 1 m / s
     &          SpeedOfLight_SI                 = 299792458.0D0
      end module physconst
