!
! pointcharges_LiF --- generate point charges in MOLPRO format for a LiF crystal
! Christian Buth
! 22.10.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 23.10.2004 (Classification of nearestness of neighbours.  Generates
!       point charges arranged spherically around and origin cell for LiF (fcc) crystals)
!


      program pointcharges_LiF
      use fortrantl
      use physconst
      implicit none


      type tlatticeindex
        integer i(3)
      end type tlatticeindex

      type tlatticevector
        real(dblprec) v(3)
      end type tlatticevector

      type tlatticetraf
        real(dblprec) m(3,3)
      end type tlatticetraf

      type tpointcharge
        real charge
        type(tlatticevector) pos
      end type tpointcharge


      character(2) element(100)
      character(20) charge(100)
      integer, parameter :: lno = 8
      integer numLattices, i, j, k, m, currLat, numDist, neighborcount(0:lno**3), &
     &        iswp, begnno, endnno, multiplicity(100), numPointCharges(100)
      real(dblprec) latticeconst(100), directdist(-lno:lno,-lno:lno,-lno:lno), &
     &              neighbordist(0:2*lno**3), rswp, r, minDist
      type(tlatticevector) atompos(100,8), point, &
     &          directlat(-lno:lno,-lno:lno,-lno:lno)
      type(tlatticetraf) basis(100)
      type(tpointcharge) madelungcharge(100,10000)

      ! Parameter for HF
      numLattices = 2
      latticeconst(1) = 3.990
      latticeconst(2) = 3.990
      atompos(1,1)%v = (/0.0, 0.0, 0.0/)
      atompos(2,1)%v = (/0.0, 0.0, 0.5/)
      multiplicity(1) = 8
      multiplicity(2) = 4
      charge(1) = 'qF'
      charge(2) = 'qLi'
      element(1) = 'F'
      element(2) = 'Li'
      r = latticeconst(1) / 2
      basis(1)%m(1:3,1) = (/0D0,   r,   r/)
      basis(1)%m(1:3,2) = (/  r, 0D0,   r/)
      basis(1)%m(1:3,3) = (/  r,   r, 0D0/)
      basis(2) = basis(1)

      ! Convert fractional atomic positions into absolut displacements
      do i=1, numLattices
        do j=1, multiplicity(i)
          atompos(i,j)%v = atompos(i,j)%v * latticeconst(i)
        end do
      end do

      ! Generate ghost atoms
      atompos(1,2)%v = basis(1)%m(1:3,1)
      atompos(1,3)%v = basis(1)%m(1:3,2)
      atompos(1,4)%v = basis(1)%m(1:3,3)
      atompos(1,5)%v = basis(1)%m(1:3,1) + basis(1)%m(1:3,2)
      atompos(1,6)%v = basis(1)%m(1:3,1) + basis(1)%m(1:3,3)
      atompos(1,7)%v = basis(1)%m(1:3,2) + basis(1)%m(1:3,3)
      atompos(1,8)%v = basis(1)%m(1:3,1) + basis(1)%m(1:3,2) + basis(1)%m(1:3,3)

      atompos(2,2)%v = atompos(2,1)%v + basis(2)%m(1:3,1)
      atompos(2,3)%v = atompos(2,1)%v + basis(2)%m(1:3,2)
      atompos(2,4)%v = atompos(2,1)%v + basis(2)%m(1:3,1) + basis(2)%m(1:3,2)

      ! Classify lattice points
      neighborcount = 0
      neighborcount(0) = 1
      neighbordist  = 0
      neighbordist(0)  = 0
      numDist = 0
      do i=-lno, lno
        do j=-lno, lno
          innermost: do k=-lno, lno
            directlat(i,j,k)%v = i * basis(1)%m(1:3,1) + j * basis(1)%m(1:3,2) &
     &                           + k * basis(1)%m(1:3,3)
            directdist(i,j,k)  = sqrt(dot_product(directlat(i,j,k)%v, &
     &                                            directlat(i,j,k)%v))

            if (abs(directdist(i,j,k)) < 1.0D-6) then
                cycle innermost
            end if

            do m=1, numDist
              if (abs(directdist(i,j,k) - neighbordist(m)) < 1.0D-3) then
                neighborcount(m) = neighborcount(m) + 1
                cycle innermost
              end if
            end do

            numDist = numDist + 1
            neighborcount(numDist) = 1
            neighbordist(numDist)  = directdist(i,j,k)
          end do innermost
        end do
      end do

      ! Sort distances (bubble sort algorithm)
      do i=1, numDist
        do j=i+1, numDist
          if (neighbordist(i) > neighbordist(j)) then
            rswp = neighbordist(i)
            neighbordist(i) = neighbordist(j)
            neighbordist(j) = rswp
            iswp = neighborcount(i)
            neighborcount(i) = neighborcount(j)
            neighborcount(j) = iswp
          end if
        end do
      end do

      ! Find minimum distance still in the parallelepiped
      minDist = huge(minDist)
      do i=-lno, lno
        do j=-lno, lno
!          r = min( directdist(    i, -lno,    1), directdist(    i,  lno,    1), &
!     &             directdist(    i,    1, -lno), directdist(    i,    1,  lno), &
!     &             directdist( -lno,    i,    1), directdist(  lno,    i,    1), &
!     &             directdist(    1,    i, -lno), directdist(    1,    i,  lno), &
!     &             directdist( -lno,    1,    i), directdist(  lno,    1,    i), &
!     &             directdist(    1, -lno,    i), directdist(    1,  lno,    i) )

          r = min( directdist(    i,    j,  lno),  &
     &             directdist(    i,    j, -lno),  &
     &             directdist(  lno,    i,    j),  &
     &             directdist( -lno,    i,    j),  &
     &             directdist(    i,  lno,    j),  &
     &             directdist(    i, -lno,    j) )
          if (r < mindist) then
            minDist = r
          end if
        end do
      end do

      write (*,*) 'There are ', numDist, ' different neighbours'
      write (*,*) 'The maximum classification is up to a distance ', minDist

      j = 0
      do i=0, min(20, numDist)
        j = j + neighborcount(i)
        write (*,*) i, neighborcount(i), j, neighbordist(i)
      end do

      ! Write geometry header
!      write (*,'(A)') '!'
!      write (*,'(A)') '! MOLPRO1992 geometry'
!      write (*,'(A)') '!'
!      write (*,*)
!      write (*,'(A)') 'int'
!      write (*,'(A)') 'nosym'

      begnno = 4
      endnno = 2

      do i=-lno, lno
        do j=-lno, lno
          do k=-lno, lno
!          displace%v = (/latticeconst(currLat), 0.0D0, 0.0D0/)
!          displace%v = displace%v  * real(i, dblprec)
!          point%v = atompos(currLat)%v + displace%v

!            if (neighbordist(nbrno) + 1.0D-4 > directdist(i,j,k)) then
!              point%v = directdist(i,j,k)
!              do currLat=1, numLattices
!                write (*,'(A,I0,3A,3(F13.8,A),3(A,I0),A)') ' a', currLat, ',', &
!     &                trim(element(currLat)), ',', point%v(1), '*a,',  &
!     &                point%v(2), '*a,', point%v(3), '*a     ! Cell ', &
!     &                'R = (', i, ', ', j, ', ', k, ')'
!              end do
!            end if

          end do
        end do
      end do

      ! Write point charge header
      write (*,*)
      write (*,*)
      write (*,'(A)') '!'
      write (*,'(A)') '! MOLPRO1992 point charges'
      write (*,'(A)') '!'
      write (*,*)

      call madelung(.false.)

      do currLat=1, numLattices
        do i=1, numPointCharges(currLat)
          point = madelungcharge(currLat, i)%pos
          point%v = point%v * AngstroemToBohr

          write (*,'(A,3(F13.8,A),2A,F5.3)') ' lat, ', point%v(1)   &
     &          , ',', point%v(2),  &
     &          ',', point%v(3), ', ',                 &
     &          trim(charge(currLat)), '*', madelungcharge(currLat, i)%charge
        end do
      end do


!
! Internal procedures
!

      contains


!
! atomsvecs --- generate absolut positions of atoms in a unit cell
!

      subroutine atomsvecs(atmcell, i, j, k)
      implicit none

      integer i, j, k, ano, amul
      type(tlatticevector) atmcell(:,:), point

      do ano=1, numLattices
        do amul=1, multiplicity(ano)
          point%v = directlat(i,j,k)%v + atompos(ano, amul)%v
          atmcell(ano, amul) = point
        end do
      end do
      end subroutine atomsvecs


!
! madelung --- find point charges arranged either spherically or like a parallelepiped around
!       a cluster containing n-th nearest neighbours
!

      subroutine madelung(spherical)
      implicit none

      logical spherical
      integer i, j, k, ano, amul, currCharge, forbidden, currForb
      type(tlatticevector) atmcell(100,8), forbiddenvecs(100,10000), point

      ! Init point charges
      numPointCharges = 0
      do i=1, numLattices
        do j=1, 10000
          madelungcharge(i, j)%charge = 0
          madelungcharge(i, j)%pos%v  = 0
        end do
      end do

      ! Compile forbidden list with points not allowed for point charges
      forbidden = 0
      do i=-lno, lno
        do j=-lno, lno
          do k=-lno, lno
            if (neighbordist(begnno) - 1.0D-4 > directdist(i,j,k)) then
              forbidden = forbidden + 1
              do ano=1, numLattices
                point%v = directlat(i,j,k)%v + atompos(ano, 1)%v
                forbiddenvecs(ano,forbidden) = point
              end do
            end if
          end do
        end do
      end do
      write (*,*) 'Number of forbidden points ', forbidden

      ! How many neighbours of the cell are in the cluster
      do i=-lno+1, lno-1
        do j=-lno+1, lno-1
          do k=-lno+1, lno-1

            ! Find unit cells up to n-th nearest neighbours
            if (.not. spherical .or. cellinside(i, j, k)) then
              call atomsvecs(atmcell, i, j, k)

              do ano=1, numLattices
                donext: do amul=1, multiplicity(ano)

                  ! Does this pointcharge already exist?
                  do currCharge=1, numPointCharges(ano)
                    point%v = atmcell(ano, amul)%v - madelungcharge(ano, currCharge)%pos%v
                    if (sqrt(dot_product(point%v, point%v)) < 1.0D-6) then
                      madelungcharge(ano, currCharge)%charge = madelungcharge(ano, currCharge)%charge &
     &                                  + 1.0 / real(multiplicity(ano))
                      cycle donext
                    end if
                  end do

                  ! Do not create point charges on forbidden points
                  do currForb=1, forbidden
                    point%v = forbiddenvecs(ano, currForb)%v - atmcell(ano, amul)%v
                    if (sqrt(dot_product(point%v, point%v)) < 1.0D-6) then
!                        write (*,*) 'forbidden!'
                      cycle donext
                    end if
                  end do

                  ! Create new point charge
                  numPointCharges(ano) = numPointCharges(ano) + 1
                  madelungcharge(ano, numPointCharges(ano))%pos%v  = atmcell(ano, amul)%v
                  madelungcharge(ano, numPointCharges(ano))%charge = 1.0 / real(multiplicity(ano))

                end do donext
              end do

            end if
          end do
        end do
      end do

      write (*,*) 'Number of point charges ', (numPointCharges(i), i=1, numLattices)
      end subroutine madelung


!
! cellinside --- test whether a unit cell is in a sphere
!

      logical function cellinside(i, j, k)
      implicit none

      integer i, j, k

      cellinside = neighbordist(begnno) - 1.0D-4 < directdist(i,j,k) .and. &
     &             neighbordist(endnno) + 1.0D-4 > directdist(i,j,k)
      end function cellinside
      end program pointcharges_LiF
